# sage_setup: distribution = sagemath-polyhedra
r"""
Triangulations of a point configuration

A point configuration is a finite set of points in Euclidean space or,
more generally, in projective space. A triangulation is a simplicial
decomposition of the convex hull of a given point configuration such
that all vertices of the simplices end up lying on points of the
configuration. That is, there are no new vertices apart from the
initial points.

Note that points that are not vertices of the convex hull need not be
used in the triangulation. A triangulation that does make use of all
points of the configuration is called fine, and you can restrict
yourself to such triangulations if you want. See
:class:`PointConfiguration` and
:meth:`~PointConfiguration.restrict_to_fine_triangulations` for
more details.

Finding a single triangulation and listing all connected
triangulations is implemented natively in this package. However, for
more advanced options [TOPCOM]_ needs to be installed; see :ref:`spkg_topcom`.

.. NOTE::

    TOPCOM and the internal algorithms tend to enumerate
    triangulations in a different order. This is why we always
    explicitly specify the engine as ``engine='topcom'`` or
    ``engine='internal'`` in the doctests. In your own applications,
    you do not need to specify the engine. By default, TOPCOM is used
    if it is available and the internal algorithms are used otherwise.

EXAMPLES:

First, we select the internal implementation for enumerating
triangulations::

    sage: PointConfiguration.set_engine('internal')   # to make doctests independent of TOPCOM

A 2-dimensional point configuration::

    sage: p = PointConfiguration([[0,0], [0,1], [1,0], [1,1], [-1,-1]]); p
    A point configuration in affine 2-space over Integer Ring consisting
    of 5 points. The triangulations of this point configuration are
    assumed to be connected, not necessarily fine, not necessarily regular.

.. PLOT::
    :width: 300 px

    p = PointConfiguration([[-1,-1], [1,1], [1,0], [0,1], [0,0]])
    sphinx_plot(p.plot(axes=False))

A triangulation of it::

    sage: t = p.triangulate(); t  # a single triangulation
    (<1,3,4>, <2,3,4>)
    sage: len(t)
    2
    sage: t[0]
    (1, 3, 4)
    sage: t[1]
    (2, 3, 4)
    sage: list(t)
    [(1, 3, 4), (2, 3, 4)]
    sage: t.plot(axes=False)                                                       # needs sage.plot
    Graphics object consisting of 12 graphics primitives

.. PLOT::
    :width: 300 px

    p = PointConfiguration([[0,0], [0,1], [1,0], [1,1], [-1,-1]])
    t = p.triangulate()
    sphinx_plot(t.plot(axes=False))

List triangulations of it::

    sage: list(p.triangulations())
    [(<1,3,4>, <2,3,4>),
     (<0,1,3>, <0,1,4>, <0,2,3>, <0,2,4>),
     (<1,2,3>, <1,2,4>),
     (<0,1,2>, <0,1,4>, <0,2,4>, <1,2,3>)]
    sage: p_fine = p.restrict_to_fine_triangulations(); p_fine
    A point configuration in affine 2-space over Integer Ring consisting
    of 5 points. The triangulations of this point configuration are
    assumed to be connected, fine, not necessarily regular.
    sage: list(p_fine.triangulations())
    [(<0,1,3>, <0,1,4>, <0,2,3>, <0,2,4>),
     (<0,1,2>, <0,1,4>, <0,2,4>, <1,2,3>)]

A 3-dimensional point configuration::

    sage: p = [[0,-1,-1], [0,0,1], [0,1,0], [1,-1,-1], [1,0,1], [1,1,0]]
    sage: points = PointConfiguration(p)
    sage: triang = points.triangulate()
    sage: triang.plot(axes=False)                                                 # needs sage.plot
    Graphics3d Object

.. PLOT::
    :width: 300 px

    p = [[0,-1,-1], [0,0,1], [0,1,0], [1,-1,-1], [1,0,1], [1,1,0]]
    points = PointConfiguration(p)
    triang = points.triangulate()
    sphinx_plot(triang.plot(axes=False))

The standard example of a non-regular triangulation (requires TOPCOM)::

    sage: # optional - topcom
    sage: PointConfiguration.set_engine('topcom')
    sage: p = PointConfiguration([[-1,-5/9], [0,10/9], [1,-5/9],
    ....:                         [-2,-10/9], [0,20/9], [2,-10/9]])
    sage: p_regular = p.restrict_to_regular_triangulations(True)
    sage: regular = p_regular.triangulations_list()
    sage: p_nonregular = p.restrict_to_regular_triangulations(False)
    sage: nonregular = p_nonregular.triangulations_list()
    sage: len(regular)
    16
    sage: len(nonregular)
    2
    sage: nonregular[0].plot(aspect_ratio=1, axes=False)                          # needs sage.plot
    Graphics object consisting of 25 graphics primitives
    sage: PointConfiguration.set_engine('internal')   # to make doctests independent of TOPCOM

Note that the points need not be in general position. That is, the
points may lie in a hyperplane and the linear dependencies will be
removed before passing the data to TOPCOM which cannot handle it::

    sage: points = [[0,0,0,1], [0,3,0,1], [3,0,0,1], [0,0,1,1],
    ....:           [0,3,1,1], [3,0,1,1], [1,1,2,1]]
    sage: points = [p + [1,2,3] for p in points]
    sage: pc = PointConfiguration(points)
    sage: pc.ambient_dim()
    7
    sage: pc.dim()
    3
    sage: pc.triangulate()
    (<0,1,2,6>, <0,1,3,6>, <0,2,3,6>, <1,2,4,6>, <1,3,4,6>, <2,3,5,6>, <2,4,5,6>)
    sage: _ in pc.triangulations()
    True
    sage: len(pc.triangulations_list())
    26

AUTHORS:

- Volker Braun: initial version, 2010

- Josh Whitney: added functionality for computing
  volumes and secondary polytopes of PointConfigurations

- Marshall Hampton: improved documentation and doctest coverage

- Volker Braun: rewrite using Parent/Element and categories. Added
  a Point class. More doctests. Less zombies.

- Volker Braun: Cythonized parts of it, added a C++ implementation
  of the bistellar flip algorithm to enumerate all connected
  triangulations.

- Volker Braun 2011: switched the triangulate() method to the
  placing triangulation (faster).
"""

########################################################################
# Note: The doctests that make use of TOPCOM are
#       marked # optional - topcom
#       If you have it installed, run doctests as
#
#   sage -tp 4 --long --optional=sage,topcom sage/geometry/triangulation/
########################################################################


########################################################################
#       Copyright (C) 2010 Volker Braun <vbraun.name@gmail.com>
#       Copyright (C) 2010 Josh Whitney <josh.r.whitney@gmail.com>
#       Copyright (C) 2010 Marshall Hampton <hamptonio@gmail.com>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#
#                  https://www.gnu.org/licenses/
########################################################################

import itertools
from copy import copy
import sys

from sage.features import FeatureNotPresentError
from sage.features.topcom import TOPCOMExecutable
from sage.matrix.constructor import matrix
from sage.misc.cachefunc import cached_method
from sage.modules.free_module_element import vector
from sage.rings.integer_ring import ZZ
from sage.rings.rational_field import QQ
from sage.structure.unique_representation import UniqueRepresentation

from sage.geometry.triangulation.base import \
    PointConfiguration_base, Point, ConnectedTriangulationsIterator

from sage.geometry.triangulation.element import Triangulation


########################################################################
class PointConfiguration(UniqueRepresentation, PointConfiguration_base):
    """
    A collection of points in Euclidean (or projective) space.

    This is the parent class for the triangulations of the point
    configuration. There are a few options to specifically select what
    kind of triangulations are admissible.

    INPUT:

    The constructor accepts the following arguments:

    - ``points`` -- the points; technically, any iterable of iterables
      will do. In particular, a :class:`PointConfiguration` can be passed.

    - ``projective`` -- boolean (default: ``False``); whether the
      point coordinates should be interpreted as projective (``True``)
      or affine (``False``) coordinates. If necessary, points are
      projectivized by setting the last homogeneous coordinate to one
      and/or affine patches are chosen internally.

    - ``connected`` -- boolean (default: ``True``); whether the
      triangulations should be connected to the regular triangulations
      via bistellar flips. These are much easier to compute than all
      triangulations.

    - ``fine`` -- boolean (default: ``False``); whether the
      triangulations must be fine, that is, make use of all points of
      the configuration

    - ``regular`` -- boolean or ``None`` (default: ``None``); whether
      the triangulations must be regular. A regular triangulation is
      one that is induced by a piecewise-linear convex support
      function. In other words, the shadows of the faces of a
      polyhedron in one higher dimension.

      * ``True``: Only regular triangulations.

      * ``False``: Only non-regular triangulations.

      * ``None`` (default): Both kinds of triangulation.

    - ``star`` -- either ``None`` or a point; whether the
      triangulations must be star. A triangulation is star if all
      maximal simplices contain a common point. The central point can
      be specified by its index (an integer) in the given points or by
      its coordinates (anything iterable.)

    EXAMPLES::

        sage: p = PointConfiguration([[0,0], [0,1], [1,0], [1,1], [-1,-1]]); p
        A point configuration in affine 2-space over Integer Ring
        consisting of 5 points. The triangulations of this point
        configuration are assumed to be connected, not necessarily fine,
        not necessarily regular.
        sage: p.triangulate()  # a single triangulation
        (<1,3,4>, <2,3,4>)
    """

    # we cache the output of _have_TOPCOM() in this class variable
    _have_TOPCOM_cached = None

    # whether to use TOPCOM. Will be set to True or False during
    # initialization. All implementations should check this boolean
    # variable to decide whether to call TOPCOM or not
    _use_TOPCOM = None

    @classmethod
    def _have_TOPCOM(cls):
        r"""
        Return whether TOPCOM is installed.

        EXAMPLES::

            sage: PointConfiguration._have_TOPCOM()    # optional - topcom
            True
        """
        if PointConfiguration._have_TOPCOM_cached is not None:
            return PointConfiguration._have_TOPCOM_cached

        try:
            import pexpect
        except ImportError:
            return False

        try:
            out = next(PointConfiguration._TOPCOM_exec('points2placingtriang',
                                                  '[[0,1],[1,1]]', verbose=False))
            PointConfiguration._have_TOPCOM_cached = True
            assert out == '{{0,1}}',\
                'TOPCOM ran but did not produce the correct output!'
        except (FeatureNotPresentError, pexpect.ExceptionPexpect):
            PointConfiguration._have_TOPCOM_cached = False

        PointConfiguration.set_engine('auto')
        return PointConfiguration._have_TOPCOM_cached

    @staticmethod
    def __classcall__(cls, points, projective=False, connected=True, fine=False, regular=None, star=None):
        r"""
        Normalize the constructor arguments to be unique keys.

        EXAMPLES::

            sage: pc1 = PointConfiguration([[1,2], [2,3], [3,4]], connected=True)
            sage: pc2 = PointConfiguration(((1,2), (2,3), (3,4)), regular=None)
            sage: pc1 is pc2   # indirect doctest
            True
        """
        if isinstance(points, PointConfiguration_base):
            pc = points
            points = tuple( p.projective() for p in points )
            projective = True
            defined_affine = pc.is_affine()
        elif projective:
            points = tuple( tuple(p) for p in points )
            defined_affine = False
        else:
            points = tuple( tuple(p)+(1,) for p in points )
            defined_affine = True
        if star is not None and star not in ZZ:
            star_point = tuple(star)
            if len(star_point) < len(points[0]):
                star_point = tuple(star)+(1,)
            star = points.index(star_point)
        return super().__classcall__(cls, points, connected, fine,
                                     regular, star, defined_affine)

    def __init__(self, points, connected, fine, regular, star, defined_affine):
        """
        Initialize a :class:`PointConfiguration` object.

        EXAMPLES::

            sage: p = PointConfiguration([[0,4], [2,3], [3,2], [4,0], [3,-2], [2,-3],
            ....:                         [0,-4], [-2,-3], [-3,-2], [-4,0], [-3,2], [-2,3]])
            sage: len(p.triangulations_list())  # long time (26s on sage.math, 2012)
            16796

        TESTS::

            sage: TestSuite(p).run()
        """
        # first, test if we have TOPCOM and set up class variables accordingly
        PointConfiguration._have_TOPCOM()

        assert connected in [True, False], 'Unknown value: connected=' + str(connected)
        self._connected = connected
        if not connected and not PointConfiguration._have_TOPCOM():
            raise ValueError('You must install TOPCOM to find non-connected triangulations.')

        assert fine in [True, False], 'Unknown value: fine=' + str(fine)
        self._fine = fine

        assert regular in [True, False, None], 'Unknown value: regular=' + str(regular)
        self._regular = regular
        if regular is not None and not PointConfiguration._have_TOPCOM():
            raise ValueError('You must install TOPCOM to test for regularity.')

        assert star is None or star in ZZ, 'Unknown value: fine=' + str(star)
        self._star = star

        PointConfiguration_base.__init__(self, points, defined_affine)

    @classmethod
    def set_engine(cls, engine='auto'):
        r"""
        Set the engine used to compute triangulations.

        INPUT:

        - ``engine`` -- either ``'auto'`` (default), ``'internal'``, or
          ``'topcom'``. The latter two instruct this package to always use
          its own triangulation algorithms or TOPCOM's algorithms,
          respectively. By default (``'auto'``), internal routines are used.

        EXAMPLES::

            sage: # optional - topcom
            sage: p = PointConfiguration([[0,0], [0,1], [1,0], [1,1], [-1,-1]])
            sage: p.set_engine('internal')   # to make doctests independent of TOPCOM
            sage: p.triangulate()
            (<1,3,4>, <2,3,4>)
            sage: p.set_engine('topcom')
            sage: p.triangulate()
            (<0,1,2>, <0,1,4>, <0,2,4>, <1,2,3>)
            sage: p.set_engine('internal')
        """
        engine = engine.lower()
        if engine not in ['auto', 'topcom', 'internal']:
            raise ValueError('Unknown value for "engine": '+str(engine))

        PointConfiguration._use_TOPCOM = (engine == 'topcom')

    def star_center(self):
        r"""
        Return the center used for star triangulations.

        .. SEEALSO:: :meth:`restrict_to_star_triangulations`.

        OUTPUT:

        A :class:`~sage.geometry.triangulation.base.Point` if a
        distinguished star central point has been fixed.
        :exc:`ValueError` exception is raised otherwise.

        EXAMPLES::

            sage: pc = PointConfiguration([(1,0), (-1,0), (0,1), (0,2)], star=(0,1)); pc
            A point configuration in affine 2-space over Integer Ring
            consisting of 4 points. The triangulations of this point
            configuration are assumed to be connected, not necessarily
            fine, not necessarily regular, and star with center P(0, 1).
            sage: pc.star_center()
            P(0, 1)

            sage: pc_nostar = pc.restrict_to_star_triangulations(None); pc_nostar
            A point configuration in affine 2-space over Integer Ring
            consisting of 4 points. The triangulations of this point
            configuration are assumed to be connected, not necessarily
            fine, not necessarily regular.
            sage: pc_nostar.star_center()
            Traceback (most recent call last):
            ...
            ValueError: The point configuration has no star center defined.
        """
        if self._star is None:
            raise ValueError('The point configuration has no star center defined.')
        else:
            return self[self._star]

    def __reduce__(self):
        r"""
        Override __reduce__ to correctly pickle/unpickle.

        TESTS::

            sage: p = PointConfiguration([[0, 1], [0, 0], [1, 0], [1,1]])
            sage: loads(p.dumps()) is p
            True

            sage: p = PointConfiguration([[0, 1, 1], [0, 0, 1], [1, 0, 1], [1,1, 1]],
            ....:                        projective=True)
            sage: loads(p.dumps()) is p
            True
        """
        if self.is_affine():
            points = tuple( p.affine() for p in self )
            return (PointConfiguration, (points, False,
                                         self._connected, self._fine, self._regular, self._star))
        else:
            points = tuple( p.projective() for p in self )
            return (PointConfiguration, (points, True,
                                         self._connected, self._fine, self._regular, self._star))

    def an_element(self):
        """
        Synonymous for :meth:`triangulate`.

        TESTS::

            sage: p = PointConfiguration([[0, 1], [0, 0], [1, 0], [1,1]])
            sage: p.an_element()
            (<0,1,3>, <1,2,3>)
        """
        return self.triangulate()

    def _element_constructor_(self, e):
        """
        Construct a triangulation.

        TESTS::

            sage: p = PointConfiguration([[0, 1], [0, 0], [1, 0], [1,1]])
            sage: p._element_constructor_([(0,1,2), (2,3,0)])
            (<0,1,2>, <0,2,3>)
        """
        return self.element_class(e, parent=self)

    Element = Triangulation

    def __iter__(self):
        """
        Iterate through the points of the point configuration.

        OUTPUT:

        Returns projective coordinates of the points. See also the
        ``PointConfiguration.points()`` method, which returns affine
        coordinates.

        EXAMPLES::

            sage: p = PointConfiguration([[1,1], [2,2], [3,3]])
            sage: list(p)     # indirect doctest
            [P(1, 1), P(2, 2), P(3, 3)]
            sage: [p[i] for i in range(p.n_points())]
            [P(1, 1), P(2, 2), P(3, 3)]
            sage: list(p.points())
            [P(1, 1), P(2, 2), P(3, 3)]
            sage: [p.point(i) for i in range(p.n_points())]
            [P(1, 1), P(2, 2), P(3, 3)]
        """
        yield from self.points()

    def _repr_(self):
        r"""
        Return a string representation.

        TESTS::

            sage: p = PointConfiguration([[1,1,1], [-1,1,1], [1,-1,1], [-1,-1,1], [1,1,-1],
            ....:                         [-1,1,-1], [1,-1,-1], [-1,-1,-1], [0,0,0]])
            sage: p._repr_()
            'A point configuration in affine 3-space over Integer Ring
            consisting of 9 points. The triangulations of this point
            configuration are assumed to be connected, not necessarily
            fine, not necessarily regular.'

            sage: PointConfiguration([[1, 1, 1], [-1, 1, 1], [1, -1, 1], [-1, -1, 1]],
            ....:                    projective=True)
            A point configuration in projective 2-space over Integer
            Ring consisting of 4 points. The triangulations of this
            point configuration are assumed to be connected,
            not necessarily fine, not necessarily regular.
        """
        s = 'A point configuration in'
        if self.is_affine():
            s += ' affine'
        else:
            s += ' projective'
        s += " %s-space over %s" % (self.ambient_dim(),self.base_ring())
        if len(self) == 1:
            s += ' consisting of '+str(len(self))+' point. '
        else:
            s += ' consisting of '+str(len(self))+' points. '

        s += 'The triangulations of this point configuration are assumed to be'

        if self._connected:
            s += ' connected,'
        else:
            s += ' not necessarily connected,'

        if self._fine:
            s += ' fine,'
        else:
            s += ' not necessarily fine,'

        if self._regular:
            s += ' regular'
        elif self._regular is False: # may be False or None, with different meanings
            s += ' irregular'
        else:
            s += ' not necessarily regular'

        if self._star is None:
            s += '.'
        else:
            s += ', and star with center '+str(self.star_center())+'.'
        if self.n_points() == 0:
            s = 'The pointless empty configuration'
        return s

    def _TOPCOM_points(self):
        r"""
        Convert the list of input points to a string that can be fed
        to TOPCOM.

        TESTS::

            sage: p = PointConfiguration([[1,1,1], [-1,1,1], [1,-1,1], [-1,-1,1], [1,1,-1]])
            sage: p._TOPCOM_points()
            '[[0,0,0,1],[-2,0,0,1],[0,-2,0,1],[-2,-2,0,1],[0,0,-2,1]]'
        """
        s = '['
        s += ','.join([
                '[' + ','.join(map(str,p.reduced_projective())) + ']'
                for p in self ])
        s += ']'
        return s

    @classmethod
    def _TOPCOM_exec(cls, executable, input_string, verbose=True):
        r"""
        Run TOPCOM.

        INPUT:

        - ``executable`` -- string; the name of the executable

        - ``input_string`` -- string; will be piped into the running
          executable's stdin

        - ``verbose`` -- boolean; whether to print out the TOPCOM
          interaction

        TESTS::

            sage: p = PointConfiguration([[1,1,1], [-1,1,1], [1,-1,1], [-1,-1,1], [1,1,-1]])
            sage: out = p._TOPCOM_exec('points2placingtriang',
            ....:                      '[[0,0,0,1],[-2,0,0,1],[0,-2,0,1],[-2,-2,0,1],[0,0,-2,1]]',
            ....:                      verbose=True)
            sage: list(out)       # optional - topcom
            #### TOPCOM input ####
            # points2placingtriang
            # [[0,0,0,1],[-2,0,0,1],[0,-2,0,1],[-2,-2,0,1],[0,0,-2,1]]
            #### TOPCOM output ####
            # {{0,1,2,4},{1,2,3,4}}
            #######################
            ['{{0,1,2,4},{1,2,3,4}}']
        """
        import pexpect

        timeout = 600
        executable_name, *args = executable.split()
        executable_absname = TOPCOMExecutable(executable_name).absolute_filename()
        proc = pexpect.spawn(executable_absname, args, timeout=timeout)
        proc.expect(r'Evaluating Commandline Options \.\.\.')
        proc.expect(r'\.\.\. done\.')
        proc.setecho(0)
        assert proc.readline().strip() == b''

        if verbose:
            print("#### TOPCOM input ####")
            print("# " + executable)
            print("# " + input_string)
            sys.stdout.flush()

        proc.send(input_string)
        proc.send('X\nX\n')

        if verbose:
            print("#### TOPCOM output ####")
            sys.stdout.flush()

        while True:
            try:
                line = proc.readline().strip()
                if not isinstance(line, str):
                    line = line.decode()
            except pexpect.TIMEOUT:
                if verbose:
                    print('# Still running ' + str(executable))
                continue
            if len(line) == 0:  # EOF
                break
            if verbose:
                print("# " + line)
                sys.stdout.flush()

            try:
                yield line.strip()
            except GeneratorExit:
                proc.close(force=True)
                return

        if verbose:
            print("#######################")
            sys.stdout.flush()

    def _TOPCOM_communicate(self, executable, verbose=True):
        r"""
        Execute TOPCOM and parse the output into a
        :class:`~sage.geometry.triangulation.element.Triangulation`.

        TESTS::

            sage: p = PointConfiguration([[1,1,1], [-1,1,1], [1,-1,1], [-1,-1,1], [1,1,-1]])
            sage: out = p._TOPCOM_communicate('points2placingtriang', verbose=True)
            sage: list(out)       # optional - topcom
            #### TOPCOM input ####
            # points2placingtriang
            # [[0,0,0,1],[-2,0,0,1],[0,-2,0,1],[-2,-2,0,1],[0,0,-2,1]]
            #### TOPCOM output ####
            # {{0,1,2,4},{1,2,3,4}}
            #######################
            [(<0,1,2,4>, <1,2,3,4>)]
        """
        for line in self._TOPCOM_exec(executable,
                                      self._TOPCOM_points(), verbose):
            triangulation = line[ line.find('{{')+2 : line.rfind('}}') ]
            triangulation = triangulation.split('},{')
            triangulation = [ [ QQ(t) for t in triangle.split(',') ]
                              for triangle in triangulation ]

            if self._star is not None:
                o = self._star
                if not all( t.count(o) > 0 for t in triangulation):
                    continue

            yield self(triangulation)

    def _TOPCOM_triangulations(self, verbose=True):
        r"""
        Return all triangulations satisfying the restrictions imposed.

        EXAMPLES::

            sage: p = PointConfiguration([[0,0], [0,1], [1,0], [1,1], [-1,-1]])
            sage: iter = p._TOPCOM_triangulations(verbose=True)
            sage: next(iter)     # optional - topcom
            #### TOPCOM input ####
            # points2triangs
            # [[0,0,1],[0,1,1],[1,0,1],[1,1,1],[-1,-1,1]]
            #### TOPCOM output ####
            # T[0] := {{0,1,2},{0,1,4},{0,2,4},{1,2,3}};
            (<0,1,2>, <0,1,4>, <0,2,4>, <1,2,3>)
        """
        command = 'points2'

        if not self._connected:
            command += 'all'

        if self._fine:
            command += 'fine'

        command += 'triangs'

        if self._regular:
            command += ' --regular'
        if self._regular is False:
            command += ' --nonregular'

        yield from self._TOPCOM_communicate(command, verbose)

    def _TOPCOM_triangulate(self, verbose=True):
        r"""
        Return one (in no particular order) triangulation subject
        to all restrictions imposed previously.

        INPUT:

        - ``verbose`` -- boolean; whether to print out the TOPCOM
          interaction

        OUTPUT:

        A :class:`~sage.geometry.triangulation.element.Triangulation`
        satisfying all restrictions imposed. This raises a :exc:`ValueError`
        if no such triangulation exists.

        EXAMPLES::

            sage: # optional - topcom
            sage: p = PointConfiguration([[0,0], [0,1], [1,0], [1,1], [-1,-1]])
            sage: p.set_engine('topcom')
            sage: p._TOPCOM_triangulate(verbose=False)
            (<0,1,2>, <0,1,4>, <0,2,4>, <1,2,3>)
            sage: list( p.triangulate() )
            [(0, 1, 2), (0, 1, 4), (0, 2, 4), (1, 2, 3)]
            sage: p.set_engine('internal')
        """
        assert self._regular is not False, \
            'When asked for a single triangulation TOPCOM ' + \
            'always returns a regular triangulation.'

        command = "points2"
        if self._fine:
            command += "finetriang"
        else:
            command += "placingtriang"

        return next(self._TOPCOM_communicate(command, verbose))

    def restrict_to_regular_triangulations(self, regular=True):
        """
        Restrict to regular triangulations.

        NOTE:

        Regularity testing requires the optional TOPCOM package.

        INPUT:

        - ``regular`` -- ``True``, ``False``, or ``None``; whether to
          restrict to regular triangulations, irregular
          triangulations, or lift any restrictions on regularity

        OUTPUT:

        A new :class:`PointConfiguration` with the same points, but
        whose triangulations will all be regular as specified. See
        :class:`PointConfiguration` for details.

        EXAMPLES::

            sage: p = PointConfiguration([[0,0], [0,1], [1,0], [1,1], [-1,-1]]); p
            A point configuration in affine 2-space over Integer Ring
            consisting of 5 points. The triangulations of this point
            configuration are assumed to be connected, not necessarily
            fine, not necessarily regular.
            sage: len(p.triangulations_list())
            4
            sage: PointConfiguration.set_engine('topcom')
            sage: p_regular = p.restrict_to_regular_triangulations() # optional - topcom
            sage: len(p_regular.triangulations_list())               # optional - topcom
            4
            sage: p == p_regular.restrict_to_regular_triangulations(regular=None) # optional - topcom
            True
            sage: PointConfiguration.set_engine('internal')
        """
        return PointConfiguration(self,
                                  connected=self._connected,
                                  fine=self._fine,
                                  regular=regular,
                                  star=self._star)

    def restrict_to_connected_triangulations(self, connected=True):
        """
        Restrict to connected triangulations.

        NOTE:

        Finding non-connected triangulations requires the optional
        TOPCOM package.

        INPUT:

        - ``connected`` -- boolean; whether to restrict to
          triangulations that are connected by bistellar flips to the
          regular triangulations

        OUTPUT:

        A new :class:`PointConfiguration` with the same points, but
        whose triangulations will all be in the connected
        component. See :class:`PointConfiguration` for details.

        EXAMPLES::

            sage: p = PointConfiguration([[0,0], [0,1], [1,0], [1,1], [-1,-1]]); p
            A point configuration in affine 2-space over Integer Ring
            consisting of 5 points. The triangulations of this point
            configuration are assumed to be connected, not necessarily
            fine, not necessarily regular.
            sage: len(p.triangulations_list())
            4
            sage: PointConfiguration.set_engine('topcom')
            sage: p_all = p.restrict_to_connected_triangulations(connected=False)  # optional - topcom
            sage: len(p_all.triangulations_list())                                 # optional - topcom
            4
            sage: p == p_all.restrict_to_connected_triangulations(connected=True)  # optional - topcom
            True
            sage: PointConfiguration.set_engine('internal')
        """
        return PointConfiguration(self,
                                  connected=connected,
                                  fine=self._fine,
                                  regular=self._regular,
                                  star=self._star)

    def restrict_to_fine_triangulations(self, fine=True):
        """
        Restrict to fine triangulations.

        INPUT:

        - ``fine`` -- boolean; whether to restrict to fine triangulations

        OUTPUT:

        A new :class:`PointConfiguration` with the same points, but
        whose triangulations will all be fine. See
        :class:`PointConfiguration` for details.

        EXAMPLES::

            sage: p = PointConfiguration([[0,0], [0,1], [1,0], [1,1], [-1,-1]])
            sage: p
            A point configuration in affine 2-space over Integer Ring
            consisting of 5 points. The triangulations of this point
            configuration are assumed to be connected, not necessarily
            fine, not necessarily regular.

            sage: len(p.triangulations_list())
            4
            sage: p_fine = p.restrict_to_fine_triangulations()
            sage: len(p.triangulations_list())
            4
            sage: p == p_fine.restrict_to_fine_triangulations(fine=False)
            True
        """
        return PointConfiguration(self,
                                  connected=self._connected,
                                  fine=fine,
                                  regular=self._regular,
                                  star=self._star)

    def restrict_to_star_triangulations(self, star):
        """
        Restrict to star triangulations with the given point as the
        center.

        INPUT:

        - ``origin`` -- ``None`` or an integer or the coordinates of a
          point. An integer denotes the index of the central point. If
          ``None`` is passed, any restriction on the starshape will be
          removed.

        OUTPUT:

        A new :class:`PointConfiguration` with the same points, but
        whose triangulations will all be star. See
        :class:`PointConfiguration` for details.

        EXAMPLES::

            sage: p = PointConfiguration([[0,0], [0,1], [1,0], [1,1], [-1,-1]])
            sage: len(list(p.triangulations()))
            4
            sage: p_star =  p.restrict_to_star_triangulations(0)
            sage: p_star is p.restrict_to_star_triangulations((0,0))
            True
            sage: p_star.triangulations_list()
            [(<0,1,3>, <0,1,4>, <0,2,3>, <0,2,4>)]
            sage: p_newstar = p_star.restrict_to_star_triangulations(1)  # pick different origin
            sage: p_newstar.triangulations_list()
            [(<1,2,3>, <1,2,4>)]
            sage: p == p_star.restrict_to_star_triangulations(star=None)
            True
        """
        return PointConfiguration(self,
                                  connected=self._connected,
                                  fine=self._fine,
                                  regular=self._regular,
                                  star=star)

    def triangulations(self, verbose=False):
        r"""
        Return all triangulations.

        - ``verbose`` -- boolean (default: ``False``); whether to
          print out the TOPCOM interaction, if any

        OUTPUT:

        A generator for the triangulations satisfying all the
        restrictions imposed. Each triangulation is returned as a
        :class:`~sage.geometry.triangulation.element.Triangulation` object.

        EXAMPLES::

            sage: p = PointConfiguration([[0,0], [0,1], [1,0], [1,1], [-1,-1]])
            sage: iter = p.triangulations()
            sage: next(iter)
            (<1,3,4>, <2,3,4>)
            sage: next(iter)
            (<0,1,3>, <0,1,4>, <0,2,3>, <0,2,4>)
            sage: next(iter)
            (<1,2,3>, <1,2,4>)
            sage: next(iter)
            (<0,1,2>, <0,1,4>, <0,2,4>, <1,2,3>)
            sage: p.triangulations_list()
            [(<1,3,4>, <2,3,4>),
             (<0,1,3>, <0,1,4>, <0,2,3>, <0,2,4>),
             (<1,2,3>, <1,2,4>),
             (<0,1,2>, <0,1,4>, <0,2,4>, <1,2,3>)]
            sage: p_fine = p.restrict_to_fine_triangulations()
            sage: p_fine.triangulations_list()
            [(<0,1,3>, <0,1,4>, <0,2,3>, <0,2,4>),
             (<0,1,2>, <0,1,4>, <0,2,4>, <1,2,3>)]

         Note that we explicitly asked the internal algorithm to
         compute the triangulations. Using TOPCOM, we obtain the same
         triangulations but in a different order::

            sage: # optional - topcom
            sage: p.set_engine('topcom')
            sage: iter = p.triangulations()
            sage: next(iter)
            (<0,1,2>, <0,1,4>, <0,2,4>, <1,2,3>)
            sage: next(iter)
            (<0,1,3>, <0,1,4>, <0,2,3>, <0,2,4>)
            sage: next(iter)
            (<1,2,3>, <1,2,4>)
            sage: next(iter)
            (<1,3,4>, <2,3,4>)
            sage: p.triangulations_list()
            [(<0,1,2>, <0,1,4>, <0,2,4>, <1,2,3>),
             (<0,1,3>, <0,1,4>, <0,2,3>, <0,2,4>),
             (<1,2,3>, <1,2,4>),
             (<1,3,4>, <2,3,4>)]
            sage: p_fine = p.restrict_to_fine_triangulations()
            sage: p_fine.set_engine('topcom')
            sage: p_fine.triangulations_list()
            [(<0,1,2>, <0,1,4>, <0,2,4>, <1,2,3>),
             (<0,1,3>, <0,1,4>, <0,2,3>, <0,2,4>)]
            sage: p.set_engine('internal')
        """
        if self._use_TOPCOM:
            yield from self._TOPCOM_triangulations(verbose)
        else:
            if not self._connected:
                raise ValueError('Need TOPCOM to find disconnected triangulations.')
            if (self._regular is not None):
                raise ValueError('Need TOPCOM to test for regularity.')
            ci = ConnectedTriangulationsIterator(self, star=self._star, fine=self._fine)
            for encoded_triangulation in ci:
                yield self(encoded_triangulation)

    def triangulations_list(self, verbose=False):
        r"""
        Return all triangulations.

        INPUT:

        - ``verbose`` -- boolean; whether to print out the TOPCOM
          interaction, if any

        OUTPUT:

        A list of triangulations (see
        :class:`~sage.geometry.triangulation.element.Triangulation`)
        satisfying all restrictions imposed previously.

        EXAMPLES::

            sage: p = PointConfiguration([[0,0], [0,1], [1,0], [1,1]])
            sage: p.triangulations_list()
            [(<0,1,2>, <1,2,3>), (<0,1,3>, <0,2,3>)]
            sage: list(map(list, p.triangulations_list()))
            [[(0, 1, 2), (1, 2, 3)], [(0, 1, 3), (0, 2, 3)]]
            sage: p.set_engine('topcom')
            sage: p.triangulations_list()      # optional - topcom
            [(<0,1,2>, <1,2,3>), (<0,1,3>, <0,2,3>)]
            sage: p.set_engine('internal')
        """
        return list(self.triangulations(verbose))

    def triangulate(self, verbose=False):
        r"""
        Return one (in no particular order) triangulation.

        INPUT:

        - ``verbose`` -- boolean; whether to print out the TOPCOM
          interaction, if any

        OUTPUT:

        A :class:`~sage.geometry.triangulation.element.Triangulation`
        satisfying all restrictions imposed. This raises a :exc:`ValueError`
        if no such triangulation exists.

        EXAMPLES::

            sage: p = PointConfiguration([[0,0], [0,1], [1,0], [1,1], [-1,-1]])
            sage: p.triangulate()
            (<1,3,4>, <2,3,4>)
            sage: list( p.triangulate() )
            [(1, 3, 4), (2, 3, 4)]

        Using TOPCOM yields a different, but equally good, triangulation::

            sage: # optional - topcom
            sage: p.set_engine('topcom')
            sage: p.triangulate()
            (<0,1,2>, <0,1,4>, <0,2,4>, <1,2,3>)
            sage: list(p.triangulate())
            [(0, 1, 2), (0, 1, 4), (0, 2, 4), (1, 2, 3)]
            sage: p.set_engine('internal')
        """
        if self._use_TOPCOM and self._regular is not False:
            try:
                return self._TOPCOM_triangulate(verbose)
            except StopIteration:
                # either topcom did not return a triangulation or we filtered it out
                pass

        if self._connected and not self._fine and self._regular is not False and self._star is None:
            return self.placing_triangulation()

        try:
            return next(self.triangulations(verbose))
        except StopIteration:
            # there is no triangulation
            pass
        raise ValueError('No triangulation with the required properties.')

    def convex_hull(self):
        """
        Return the convex hull of the point configuration.

        EXAMPLES::

            sage: p = PointConfiguration([[0,0], [0,1], [1,0], [1,1], [-1,-1]])
            sage: p.convex_hull()
            A 2-dimensional polyhedron in ZZ^2 defined as the convex hull of 4 vertices
        """
        try:
            return self._polyhedron
        except AttributeError:
            pass

        from sage.geometry.polyhedron.constructor import Polyhedron
        pts = [p.reduced_affine() for p in self.points()]
        self._polyhedron = Polyhedron(vertices=pts)
        return self._polyhedron

    @cached_method
    def restricted_automorphism_group(self):
        r"""
        Return the restricted automorphism group.

        First, let the linear automorphism group be the subgroup of
        the affine group `AGL(d,\RR) = GL(d,\RR) \ltimes \RR^d`
        preserving the `d`-dimensional point configuration. The
        affine group acts in the usual way `\vec{x}\mapsto
        A\vec{x}+b` on the ambient space.

        The restricted automorphism group is the subgroup of the
        linear automorphism group generated by permutations of
        points. See [BSS2009]_ for more details and a description of the
        algorithm.

        OUTPUT:

        A
        :class:`PermutationGroup<sage.groups.perm_gps.permgroup.PermutationGroup_generic>`
        that is isomorphic to the restricted automorphism group is
        returned.

        Note that in Sage, permutation groups always act on positive
        integers while lists etc. are indexed by nonnegative
        integers. The indexing of the permutation group is chosen to
        be shifted by ``+1``. That is, the transposition ``(i,j)`` in
        the permutation group corresponds to exchange of ``self[i-1]``
        and ``self[j-1]``.

        EXAMPLES::

            sage: pyramid = PointConfiguration([[1,0,0], [0,1,1], [0,1,-1],
            ....:                               [0,-1,-1], [0,-1,1]])
            sage: G = pyramid.restricted_automorphism_group()                      # needs sage.graphs sage.groups
            sage: G == PermutationGroup([[(3,5)], [(2,3),(4,5)], [(2,4)]])         # needs sage.graphs sage.groups
            True
            sage: DihedralGroup(4).is_isomorphic(G)                                # needs sage.graphs sage.groups
            True

        The square with an off-center point in the middle. Note that
        the middle point breaks the restricted automorphism group
        `D_4` of the convex hull::

            sage: square = PointConfiguration([(3/4,3/4), (1,1), (1,-1), (-1,-1), (-1,1)])
            sage: square.restricted_automorphism_group()                           # needs sage.graphs sage.groups
            Permutation Group with generators [(3,5)]
            sage: DihedralGroup(1).is_isomorphic(_)                                # needs sage.graphs sage.groups
            True
        """
        v_list = [ vector(p.projective()) for p in self ]
        Qinv = sum( v.column() * v.row() for v in v_list ).inverse()

        # construct the graph
        from sage.graphs.graph import Graph

        # Was set to sparse = False, but there is a problem with Graph
        # backends. It should probably be set back to sparse = False as soon as
        # the backends are fixed.
        G = Graph(sparse=True)
        for i in range(len(v_list)):
            for j in range(i+1,len(v_list)):
                v_i = v_list[i]
                v_j = v_list[j]
                G.add_edge(i+1,j+1, v_i * Qinv * v_j)

        return G.automorphism_group(edge_labels=True)

    def face_codimension(self, point):
        r"""
        Return the smallest `d\in\ZZ` such that ``point`` is
        contained in the interior of a codimension-`d` face.

        EXAMPLES::

            sage: triangle = PointConfiguration([[0,0], [1,-1], [1,0], [1,1]])
            sage: triangle.point(2)
            P(1, 0)
            sage: triangle.face_codimension(2)
            1
            sage: triangle.face_codimension([1,0])
            1

        This also works for degenerate cases like the tip of the
        pyramid over a square (which saturates four inequalities)::

            sage: pyramid = PointConfiguration([[1,0,0], [0,1,1], [0,1,-1],
            ....:                               [0,-1,-1], [0,-1,1]])
            sage: pyramid.face_codimension(0)
            3
        """
        try:
            p = vector(self.point(point).reduced_affine())
        except TypeError:
            p = vector(point)

        inequalities = []
        for ieq in self.convex_hull().inequality_generator():
            if (ieq.A()*p + ieq.b() == 0):
                inequalities += [ ieq.vector() ]
        return matrix(inequalities).rank()

    def face_interior(self, dim=None, codim=None):
        """
        Return points by the codimension of the containing face in the convex hull.

        EXAMPLES::

            sage: triangle = PointConfiguration([[-1,0], [0,0], [1,-1], [1,0], [1,1]])
            sage: triangle.face_interior()
            ((1,), (3,), (0, 2, 4))
            sage: triangle.face_interior(dim=0)    # the vertices of the convex hull
            (0, 2, 4)
            sage: triangle.face_interior(codim=1)  # interior of facets
            (3,)
        """
        assert not (dim is not None and codim is not None), "You cannot specify both dim and codim."

        if (dim is not None):
            return self.face_interior()[self.convex_hull().dim()-dim]
        if (codim is not None):
            return self.face_interior()[codim]

        try:
            return self._face_interior
        except AttributeError:
            pass

        d = [ self.face_codimension(i) for i in range(self.n_points()) ]

        return tuple( tuple(i for i in range(self.n_points()) if d[i] == codim )
                      for codim in range(self.dim()+1) )

    def exclude_points(self, point_idx_list):
        """
        Return a new point configuration with the given points
        removed.

        INPUT:

        - ``point_idx_list`` -- list of integers; the indices of
          points to exclude

        OUTPUT:

        A new :class:`PointConfiguration` with the given points
        removed.

        EXAMPLES::

            sage: p = PointConfiguration([[-1,0], [0,0], [1,-1], [1,0], [1,1]])
            sage: list(p)
            [P(-1, 0), P(0, 0), P(1, -1), P(1, 0), P(1, 1)]
            sage: q = p.exclude_points([3])
            sage: list(q)
            [P(-1, 0), P(0, 0), P(1, -1), P(1, 1)]
            sage: p.exclude_points(p.face_interior(codim=1)).points()
            (P(-1, 0), P(0, 0), P(1, -1), P(1, 1))
        """
        points = [self.point(i) for i in range(self.n_points())
                  if i not in point_idx_list]
        return PointConfiguration(points,
                                  projective=False,
                                  connected=self._connected,
                                  fine=self._fine,
                                  regular=self._regular,
                                  star=self._star)

    def volume(self, simplex=None):
        """
        Find `n!` times the `n`-volume of a simplex of dimension `n`.

        INPUT:

        - ``simplex`` -- (optional argument) a simplex from a
          triangulation T specified as a list of point indices

        OUTPUT:

        * If a simplex was passed as an argument: `n!` * (volume of ``simplex``).

        * Without argument: `n!` * (the total volume of the convex hull).

        EXAMPLES:

        The volume of the standard simplex should always be 1::

            sage: p = PointConfiguration([[0,0], [1,0], [0,1], [1,1]])
            sage: p.volume([0,1,2])
            1
            sage: simplex = p.triangulate()[0]  # first simplex of triangulation
            sage: p.volume(simplex)
            1

        The square can be triangulated into two minimal simplices, so
        in the "integral" normalization its volume equals two::

            sage: p.volume()
            2

        .. NOTE::

            We return `n!` * (metric volume of the simplex) to ensure that
            the volume is an integer.  Essentially, this normalizes
            things so that the volume of the standard `n`-simplex is 1.
            See [GKZ1994]_ page 182.
        """
        if (simplex is None):
            return sum([ self.volume(s) for s in self.triangulate() ])

        #Form a matrix whose columns are the points of simplex
        #with the first point of simplex shifted to the origin.
        v = [ self.point(i).reduced_affine_vector() for i in simplex ]
        m = matrix([ v_i - v[0] for v_i in v[1:] ])
        return abs(m.det())

    def secondary_polytope(self):
        r"""
        Calculate the secondary polytope of the point configuration.

        For a definition of the secondary polytope, see [GKZ1994]_ page 220
        Definition 1.6.

        Note that if you restricted the admissible triangulations of
        the point configuration then the output will be the
        corresponding face of the whole secondary polytope.

        OUTPUT:

        The secondary polytope of the point configuration as an
        instance of
        :class:`~sage.geometry.polyhedron.base.Polyhedron_base`.

        EXAMPLES::

            sage: p = PointConfiguration([[0,0], [1,0], [2,1], [1,2], [0,1]])
            sage: poly = p.secondary_polytope()
            sage: poly.vertices_matrix()
            [1 1 3 3 5]
            [3 5 1 4 1]
            [4 2 5 2 4]
            [2 4 2 5 4]
            [5 3 4 1 1]
            sage: poly.Vrepresentation()
            (A vertex at (1, 3, 4, 2, 5),
             A vertex at (1, 5, 2, 4, 3),
             A vertex at (3, 1, 5, 2, 4),
             A vertex at (3, 4, 2, 5, 1),
             A vertex at (5, 1, 4, 4, 1))
            sage: poly.Hrepresentation()
            (An equation (0, 0, 1, 2, 1) x - 13 == 0,
             An equation (1, 0, 0, 2, 2) x - 15 == 0,
             An equation (0, 1, 0, -3, -2) x + 13 == 0,
             An inequality (0, 0, 0, -1, -1) x + 7 >= 0,
             An inequality (0, 0, 0, 1, 0) x - 2 >= 0,
             An inequality (0, 0, 0, -2, -1) x + 11 >= 0,
             An inequality (0, 0, 0, 0, 1) x - 1 >= 0,
             An inequality (0, 0, 0, 3, 2) x - 14 >= 0)
        """
        from sage.geometry.polyhedron.constructor import Polyhedron
        #TODO: once restriction to regular triangulations is fixed,
        #change the next line to only take the regular triangulations,
        #since they are the vertices of the secondary polytope anyway.
        l = self.triangulations_list()
        return Polyhedron(vertices=[x.gkz_phi() for x in l])

    def circuits_support(self):
        r"""
        A generator for the supports of the circuits of the point configuration.

        See :meth:`circuits` for details.

        OUTPUT:

        A generator for the supports `C_-\cup C_+` (returned as a
        Python tuple) for all circuits of the point configuration.

        EXAMPLES::

            sage: p = PointConfiguration([(0,0), (+1,0), (-1,0), (0,+1), (0,-1)])
            sage: sorted(p.circuits_support())
            [(0, 1, 2), (0, 3, 4), (1, 2, 3, 4)]
        """
        n = len(self)
        U = [self[i].reduced_projective() for i in range(n)]

        # the index set of U
        I = set(range(n))
        # The (indices of) known independent elements of U
        independent_k = [(i,) for i in range(n)]
        supports_k = []

        for k in range(2, self.dim() + 3):

            # possibly linear dependent subsets
            supports_knext = set()
            possible_dependency = set()
            for indep in independent_k:
                indep_plus_one = [ tuple(sorted(indep+(i,))) for i in (I-set(indep)) ]
                possible_dependency.update(indep_plus_one)
            for supp in supports_k:
                supp_plus_one = [ tuple(sorted(supp+(i,))) for i in (I-set(supp)) ]
                possible_dependency.difference_update(supp_plus_one)
                supports_knext.update(supp_plus_one)

            # remember supports and independents for the next k-iteration
            supports_k = list(supports_knext)
            independent_k = []
            for idx in possible_dependency:
                rk = matrix([ U[i] for i in idx ]).rank()
                if rk == k:
                    independent_k.append(idx)
                else:
                    supports_k.append(idx)
                    yield idx
        assert independent_k == []  # there are no independent (self.dim()+3)-tuples

    def circuits(self):
        r"""
        Return the circuits of the point configuration.

        Roughly, a circuit is a minimal linearly dependent subset of
        the points. That is, a circuit is a partition

        .. MATH::

            \{ 0, 1, \dots, n-1 \} = C_+ \cup C_0 \cup C_-

        such that there is an (unique up to an overall normalization) affine
        relation

        .. MATH::

            \sum_{i\in C_+}  \alpha_i \vec{p}_i =
            \sum_{j\in C_-}  \alpha_j \vec{p}_j

        with all positive (or all negative) coefficients, where
        `\vec{p}_i=(p_1,\dots,p_k,1)` are the projective coordinates
        of the `i`-th point.

        OUTPUT:

        The list of (unsigned) circuits as triples `(C_+, C_0,
        C_-)`. The swapped circuit `(C_-, C_0, C_+)` is not returned
        separately.

        EXAMPLES::

            sage: p = PointConfiguration([(0,0), (+1,0), (-1,0), (0,+1), (0,-1)])
            sage: sorted(p.circuits())
            [((0,), (1, 2), (3, 4)), ((0,), (3, 4), (1, 2)), ((1, 2), (0,), (3, 4))]


        TESTS::

            sage: U=matrix([
            ....:    [ 0, 0, 0, 0, 0, 2, 4,-1, 1, 1, 0, 0, 1, 0],
            ....:    [ 0, 0, 0, 1, 0, 0,-1, 0, 0, 0, 0, 0, 0, 0],
            ....:    [ 0, 2, 0, 0, 0, 0,-1, 0, 1, 0, 1, 0, 0, 1],
            ....:    [ 0, 1, 1, 0, 0, 1, 0,-2, 1, 0, 0,-1, 1, 1],
            ....:    [ 0, 0, 0, 0, 1, 0,-1, 0, 0, 0, 0, 0, 0, 0]
            ....: ])
            sage: p = PointConfiguration(U.columns())
            sage: len(p.circuits())    # long time
            218
        """
        try:
            return self._circuits
        except AttributeError:
            pass

        n = len(self)
        U = [self[i].reduced_projective() for i in range(n)]

        Circuits = ()
        for support in self.circuits_support():
            m = matrix([ U[i] for i in support ]).transpose()
            ker = m.right_kernel().basis()[0]
            assert len(ker) == len(support)
            Cplus = [ support[i] for i in range(len(support)) if ker[i] > 0 ]
            Cminus = [ support[i] for i in range(len(support)) if ker[i] < 0 ]
            Czero = set( range(n) ).difference(support)
            Circuits += ( (tuple(Cplus), tuple(Czero), tuple(Cminus)), )
        self._circuits = Circuits
        return Circuits

    def positive_circuits(self, *negative):
        r"""
        Return the positive part of circuits with fixed negative part.

        A circuit is a pair `(C_+, C_-)`, each consisting of a subset
        (actually, an ordered tuple) of point indices.

        INPUT:

        - ``*negative`` -- integer; the indices of points

        OUTPUT: a tuple of all circuits with `C_-` = ``negative``

        EXAMPLES::

            sage: p = PointConfiguration([(1,0,0), (0,1,0), (0,0,1), (-2,0,-1), (-2,-1,0),
            ....:                         (-3,-1,-1), (1,1,1), (-1,0,0), (0,0,0)])
            sage: sorted(p.positive_circuits(8))
            [(0, 1, 2, 5), (0, 1, 4), (0, 2, 3), (0, 3, 4, 6), (0, 5, 6), (0, 7)]
            sage: p.positive_circuits(0,5,6)
            ((8,),)
        """
        pos = ()
        negative = tuple(sorted(negative))
        for circuit in self.circuits():
            Cpos = circuit[0]
            Cneg = circuit[2]
            if Cpos == negative:
                pos += ( Cneg, )
            elif Cneg == negative:
                pos += ( Cpos, )
        return pos

    def bistellar_flips(self):
        r"""
        Return the bistellar flips.

        OUTPUT:

        The bistellar flips as a tuple. Each flip is a pair
        `(T_+,T_-)` where `T_+` and `T_-` are partial triangulations
        of the point configuration.

        EXAMPLES::

            sage: pc = PointConfiguration([(0,0),(1,0),(0,1),(1,1)])
            sage: pc.bistellar_flips()
            (((<0,1,3>, <0,2,3>), (<0,1,2>, <1,2,3>)),)
            sage: Tpos, Tneg = pc.bistellar_flips()[0]
            sage: Tpos.plot(axes=False)                                            # needs sage.plot
            Graphics object consisting of 11 graphics primitives
            sage: Tneg.plot(axes=False)                                            # needs sage.plot
            Graphics object consisting of 11 graphics primitives

        The 3d analog::

            sage: pc = PointConfiguration([(0,0,0),(0,2,0),(0,0,2),(-1,0,0),(1,1,1)])
            sage: pc.bistellar_flips()
            (((<0,1,2,3>, <0,1,2,4>), (<0,1,3,4>, <0,2,3,4>, <1,2,3,4>)),)

        A 2d flip on the base of the pyramid over a square::

            sage: pc = PointConfiguration([(0,0,0),(0,2,0),(0,0,2),(0,2,2),(1,1,1)])
            sage: pc.bistellar_flips()
            (((<0,1,3>, <0,2,3>), (<0,1,2>, <1,2,3>)),)
            sage: Tpos, Tneg = pc.bistellar_flips()[0]
            sage: Tpos.plot(axes=False)                                            # needs sage.plot
            Graphics3d Object
        """
        flips = []
        for C in self.circuits():
            Cpos = list(C[0])
            Cneg = list(C[2])
            Tpos = [Cpos + Cneg[0:i] + Cneg[i+1:len(Cneg)]
                    for i in range(len(Cneg))]
            Tneg = [Cneg + Cpos[0:i] + Cpos[i+1:len(Cpos)]
                    for i in range(len(Cpos))]
            flips.append((self.element_class(Tpos, parent=self, check=False),
                          self.element_class(Tneg, parent=self, check=False)))
        return tuple(flips)

    def lexicographic_triangulation(self):
        r"""
        Return the lexicographic triangulation.

        The algorithm was taken from [PUNTOS]_.

        EXAMPLES::

            sage: p = PointConfiguration([(0,0), (+1,0), (-1,0), (0,+1), (0,-1)])
            sage: p.lexicographic_triangulation()
            (<1,3,4>, <2,3,4>)

        TESTS::

            sage: U = matrix([
            ....:    [ 0, 0, 0, 0, 0, 2, 4,-1, 1, 1, 0, 0, 1, 0],
            ....:    [ 0, 0, 0, 1, 0, 0,-1, 0, 0, 0, 0, 0, 0, 0],
            ....:    [ 0, 2, 0, 0, 0, 0,-1, 0, 1, 0, 1, 0, 0, 1],
            ....:    [ 0, 1, 1, 0, 0, 1, 0,-2, 1, 0, 0,-1, 1, 1],
            ....:    [ 0, 0, 0, 0, 1, 0,-1, 0, 0, 0, 0, 0, 0, 0]
            ....: ])
            sage: pc = PointConfiguration(U.columns())
            sage: pc.lexicographic_triangulation()
            (<1,3,4,7,10,13>, <1,3,4,8,10,13>, <1,3,6,7,10,13>, <1,3,6,8,10,13>,
             <1,4,6,7,10,13>, <1,4,6,8,10,13>, <2,3,4,6,7,12>, <2,3,4,7,12,13>,
             <2,3,6,7,12,13>, <2,4,6,7,12,13>, <3,4,5,6,9,12>, <3,4,5,8,9,12>,
             <3,4,6,7,11,12>, <3,4,6,9,11,12>, <3,4,7,10,11,13>, <3,4,7,11,12,13>,
             <3,4,8,9,10,12>, <3,4,8,10,12,13>, <3,4,9,10,11,12>, <3,4,10,11,12,13>,
             <3,5,6,8,9,12>, <3,6,7,10,11,13>, <3,6,7,11,12,13>, <3,6,8,9,10,12>,
             <3,6,8,10,12,13>, <3,6,9,10,11,12>, <3,6,10,11,12,13>, <4,5,6,8,9,12>,
             <4,6,7,10,11,13>, <4,6,7,11,12,13>, <4,6,8,9,10,12>, <4,6,8,10,12,13>,
             <4,6,9,10,11,12>, <4,6,10,11,12,13>)
            sage: len(_)
            34
        """
        lex_supp = set()
        for circuit in self.circuits():
            Cplus = circuit[0]
            Cminus = circuit[2]
            s0 = min(Cplus + Cminus)
            if s0 in Cplus:
                lex_supp.add(Cplus)
            else:
                lex_supp.add(Cminus)

        lex_supp = sorted(lex_supp, key=lambda x:-len(x))
        basepts = copy(lex_supp)
        for i in range(len(lex_supp)-1):
            for j in range(i+1,len(lex_supp)):
                if set(lex_supp[j]).issubset(set(lex_supp[i])):
                    try:
                        basepts.remove(lex_supp[i])
                    except ValueError:
                        pass

        basepts = [ (len(b),)+b for b in basepts ]     # decorate
        basepts = sorted(basepts)                      # sort
        basepts = [ b[1:] for b in basepts ]           # undecorate

        def make_cotriang(basepts):
            if len(basepts) == 0:
                return [frozenset()]
            triangulation = set()
            for tail in make_cotriang(basepts[1:]):
                for head in basepts[0]:
                    triangulation.update([ frozenset([head]).union(tail) ])

            nonminimal = set()
            for rel in itertools.combinations(triangulation, 2):
                if rel[0].issubset(rel[1]):
                    nonminimal.update([rel[1]])
                if rel[1].issubset(rel[0]):
                    nonminimal.update([rel[0]])
            triangulation.difference_update(nonminimal)

            triangulation = [ [len(t)]+sorted(t) for t in triangulation ] # decorate
            triangulation = sorted(triangulation)                         # sort
            triangulation = [ frozenset(t[1:]) for t in triangulation ]   # undecorate

            return triangulation

        triangulation = make_cotriang(basepts)
        I = frozenset(range(self.n_points()))
        triangulation = [ tuple(I.difference(t)) for t in triangulation ]

        return self(triangulation)

    @cached_method
    def distance_affine(self, x, y):
        r"""
        Return the distance between two points.

        The distance function used in this method is `d_{aff}(x,y)^2`,
        the square of the usual affine distance function

        .. MATH::

            d_{aff}(x,y) = |x-y|

        INPUT:

        - ``x``, ``y`` -- two points of the point configuration

        OUTPUT:

        The metric distance-square `d_{aff}(x,y)^2`. Note that this
        distance lies in the same field as the entries of ``x``,
        ``y``. That is, the distance of rational points will be
        rational and so on.

        EXAMPLES::

            sage: pc = PointConfiguration([(0,0),(1,0),(2,1),(1,2),(0,1)])
            sage: [pc.distance_affine(pc.point(0), p) for p in pc.points()]
            [0, 1, 5, 5, 1]
        """
        self._assert_is_affine()
        d = 0
        for xi, yi in zip(x.projective(), y.projective()):
            d += (xi-yi)**2
        return d

    @cached_method
    def distance_FS(self, x, y):
        r"""
        Return the distance between two points.

        The distance function used in this method is `1-\cos
        d_{FS}(x,y)^2`, where `d_{FS}` is the Fubini-Study distance of
        projective points. Recall the Fubini-Studi distance function

        .. MATH::

            d_{FS}(x,y) = \arccos \sqrt{ \frac{(x\cdot y)^2}{|x|^2 |y|^2} }

        INPUT:

        - ``x``, ``y`` -- two points of the point configuration

        OUTPUT:

        The distance `1-\cos d_{FS}(x,y)^2`. Note that this distance
        lies in the same field as the entries of ``x``, ``y``. That
        is, the distance of rational points will be rational and so
        on.

        EXAMPLES::

            sage: pc = PointConfiguration([(0,0), (1,0), (2,1), (1,2), (0,1)])
            sage: [pc.distance_FS(pc.point(0), p) for p in pc.points()]
            [0, 1/2, 5/6, 5/6, 1/2]
        """
        x2 = y2 = xy = 0
        for xi, yi in zip(x.projective(), y.projective()):
            x2 += xi*xi
            y2 += yi*yi
            xy += xi*yi
        return 1-xy*xy/(x2*y2)

    @cached_method
    def distance(self, x, y):
        """
        Return the distance between two points.

        INPUT:

        - ``x``, ``y`` -- two points of the point configuration

        OUTPUT:

        The distance between ``x`` and ``y``, measured either with
        :meth:`distance_affine` or :meth:`distance_FS` depending on
        whether the point configuration is defined by affine or
        projective points. These are related, but not equal to the
        usual flat and Fubini-Study distance.

        EXAMPLES::

            sage: pc = PointConfiguration([(0,0), (1,0), (2,1), (1,2), (0,1)])
            sage: [pc.distance(pc.point(0), p) for p in pc.points()]
            [0, 1, 5, 5, 1]

            sage: pc = PointConfiguration([(0,0,1), (1,0,1), (2,1,1), (1,2,1), (0,1,1)],
            ....:                         projective=True)
            sage: [pc.distance(pc.point(0), p) for p in pc.points()]
            [0, 1/2, 5/6, 5/6, 1/2]
        """
        if self.is_affine():
            return self.distance_affine(x,y)
        else:
            return self.distance_FS(x,y)

    def farthest_point(self, points, among=None):
        """
        Return the point with the most distance from ``points``.

        INPUT:

        - ``points`` -- list of points

        - ``among`` -- list of points or ``None`` (default); the set
          of points from which to pick the farthest one. By default,
          all points of the configuration are considered.

        OUTPUT:

        A :class:`~sage.geometry.triangulation.base.Point` with
        largest minimal distance from all given ``points``.

        EXAMPLES::

            sage: pc = PointConfiguration([(0,0), (1,0), (1,1), (0,1)])
            sage: pc.farthest_point([pc.point(0)])
            P(1, 1)
        """
        if len(points) == 0:
            return self.point(0)
        if among is None:
            among = self.points()
        p_max = None
        for p in among:
            if p in points:
                continue
            if p_max is None:
                p_max = p
                d_max = min(self.distance(p,q) for q in points)
                continue
            d = min(self.distance(p,q) for q in points)
            if d > d_max:
                p_max = p
        return p_max

    def contained_simplex(self, large=True, initial_point=None, point_order=None):
        """
        Return a simplex contained in the point configuration.

        INPUT:

        - ``large`` -- boolean; whether to attempt to return a large
          simplex

        - ``initial_point`` -- a
          :class:`~sage.geometry.triangulation.base.Point` or ``None``
          (default). A specific point to start with when picking the
          simplex vertices.

        - ``point_order`` -- list or tuple of (some or all)
          :class:`~sage.geometry.triangulation.base.Point` s or ``None``
          (default)

        OUTPUT:

        A tuple of points that span a simplex of dimension
        :meth:`dim`. If ``large==True``, the simplex is constructed by
        successively picking the farthest point. This will ensure that
        the simplex is not unnecessarily small, but will in general
        not return a maximal simplex.
        If a ``point_order`` is specified, the simplex is greedily
        constructed by considering the points in this order.
        The ``large`` option and ``initial_point`` is ignored in this case.
        The ``point_order`` may contain only a subset of the points;
        in this case, the dimension of the simplex will be the dimension of
        this subset.

        EXAMPLES::

            sage: pc = PointConfiguration([(0,0), (1,0), (2,1), (1,1), (0,1)])
            sage: pc.contained_simplex()
            (P(0, 1), P(2, 1), P(1, 0))
            sage: pc.contained_simplex(large=False)
            (P(0, 1), P(1, 1), P(1, 0))
            sage: pc.contained_simplex(initial_point=pc.point(2))
            (P(2, 1), P(0, 0), P(1, 0))

            sage: pc = PointConfiguration([[0,0], [0,1], [1,0], [1,1], [-1,-1]])
            sage: pc.contained_simplex()
            (P(-1, -1), P(1, 1), P(0, 1))
            sage: pc.contained_simplex(point_order=[pc[1], pc[3], pc[4], pc[2], pc[0]])
            (P(0, 1), P(1, 1), P(-1, -1))

        Lower-dimensional example::

            sage: pc.contained_simplex(point_order=[pc[0], pc[3], pc[4]])
            (P(0, 0), P(1, 1))

        TESTS::

            sage: pc = PointConfiguration([[0,0], [0,1], [1,0]])
            sage: pc.contained_simplex()
            (P(1, 0), P(0, 1), P(0, 0))
            sage: pc = PointConfiguration([[0,0], [0,1]])
            sage: pc.contained_simplex()
            (P(0, 1), P(0, 0))
            sage: pc = PointConfiguration([[0,0]])
            sage: pc.contained_simplex()
            (P(0, 0),)
            sage: pc = PointConfiguration([])
            sage: pc.contained_simplex()
            ()
        """
        self._assert_is_affine()
        if point_order is None:
            points = list(self.points())
        else:
            points = list(reversed(point_order))
            # points are removed one by one from the end.
            initial_point = None
            large = False
            # If point_order is specified, the points of the
            # PointConfiguration are actually ignored.
        if not points:
            return tuple()

        if initial_point is None:
            origin = points.pop()
        else:
            origin = initial_point
            points.remove(origin)
        vertices = [origin]
        edges = []
        while points and len(vertices) <= self.dim():
            if large:
                p = self.farthest_point(vertices, points)
                points.remove(p)
            else:
                p = points.pop()
            edge = p.reduced_affine_vector() - origin.reduced_affine_vector()
            if edges and (ker * edge).is_zero():
                continue
            vertices.append(p)
            edges.append(edge)
            ker = matrix(edges).right_kernel().matrix()
        return tuple(vertices)

    def placing_triangulation(self, point_order=None):
        r"""
        Construct the placing (pushing) triangulation.

        INPUT:

        - ``point_order`` -- list of points or integers. The order in
          which the points are to be placed. If not given, the points
          will be placed in some arbitrary order that attempts to
          produce a small number of simplices.

        OUTPUT: a :class:`~sage.geometry.triangulation.triangulation.Triangulation`

        EXAMPLES::

            sage: pc = PointConfiguration([(0,0), (1,0), (2,1), (1,2), (0,1)])
            sage: pc.placing_triangulation()
            (<0,1,2>, <0,2,4>, <2,3,4>)
            sage: pc.placing_triangulation(point_order=(3,2,1,4,0))
            (<0,1,4>, <1,2,3>, <1,3,4>)
            sage: pc.placing_triangulation(point_order=[pc[1], pc[3], pc[4], pc[0]])
            (<0,1,4>, <1,3,4>)
            sage: U = matrix([
            ....:    [ 0, 0, 0, 0, 0, 2, 4,-1, 1, 1, 0, 0, 1, 0],
            ....:    [ 0, 0, 0, 1, 0, 0,-1, 0, 0, 0, 0, 0, 0, 0],
            ....:    [ 0, 2, 0, 0, 0, 0,-1, 0, 1, 0, 1, 0, 0, 1],
            ....:    [ 0, 1, 1, 0, 0, 1, 0,-2, 1, 0, 0,-1, 1, 1],
            ....:    [ 0, 0, 0, 0, 1, 0,-1, 0, 0, 0, 0, 0, 0, 0]
            ....: ])
            sage: p = PointConfiguration(U.columns())
            sage: triangulation = p.placing_triangulation();  triangulation
            (<0,2,3,4,6,7>, <0,2,3,4,6,12>, <0,2,3,4,7,13>, <0,2,3,4,12,13>,
             <0,2,3,6,7,13>, <0,2,3,6,12,13>, <0,2,4,6,7,13>, <0,2,4,6,12,13>,
             <0,3,4,6,7,12>, <0,3,4,7,12,13>, <0,3,6,7,12,13>, <0,4,6,7,12,13>,
             <1,3,4,5,6,12>, <1,3,4,6,11,12>, <1,3,4,7,11,13>, <1,3,4,11,12,13>,
             <1,3,6,7,11,13>, <1,3,6,11,12,13>, <1,4,6,7,11,13>, <1,4,6,11,12,13>,
             <3,4,6,7,11,12>, <3,4,7,11,12,13>, <3,6,7,11,12,13>, <4,6,7,11,12,13>)
            sage: sum(p.volume(t) for t in triangulation)
            42
            sage: p0 = PointConfiguration([(0,0), (+1,0), (-1,0), (0,+1), (0,-1)])
            sage: p0.pushing_triangulation(point_order=[1,2,0,3,4])
            (<1,2,3>, <1,2,4>)
            sage: p0.pushing_triangulation(point_order=[0,1,2,3,4])
            (<0,1,3>, <0,1,4>, <0,2,3>, <0,2,4>)

        The same triangulation with renumbered points 0->4, 1->0, etc::

            sage: p1 = PointConfiguration([(+1,0), (-1,0), (0,+1), (0,-1), (0,0)])
            sage: p1.pushing_triangulation(point_order=[4,0,1,2,3])
            (<0,2,4>, <0,3,4>, <1,2,4>, <1,3,4>)
        """
        facet_normals = dict()

        def facets_of_simplex(simplex):
            """
            Return the facets of the simplex and store the normals in facet_normals
            """
            simplex = list(simplex)
            origin = simplex[0]
            rest = simplex[1:]
            span = matrix([origin.reduced_affine_vector()-p.reduced_affine_vector()
                           for p in rest])
            # span.inverse() linearly transforms the simplex into the unit simplex
            normals = span.inverse().columns()
            facets = []
            # The facets incident to the chosen vertex "origin"
            for opposing_vertex, normal in zip(rest, normals):
                facet = frozenset([origin] + [p for p in rest if p is not opposing_vertex])
                facets.append(facet)
                normal.set_immutable()
                facet_normals[facet] = normal
            # The remaining facet that is not incident to "origin"
            facet = frozenset(rest)
            normal = -sum(normals)
            normal.set_immutable()
            facet_normals[facet] = normal
            facets.append(facet)
            return set(facets)

        # input verification
        self._assert_is_affine()

        point_order_is_given = point_order is not None
        if point_order is None:
            point_order = list(self.points())
        elif isinstance(point_order[0], Point):
            point_order = list(point_order)
            assert all(p.point_configuration() is self for p in point_order),\
                "Some point in 'point_order' does not belong to the PointConfiguration."
        else:
            point_order = [self.point(i) for i in point_order]

        # construct the initial simplex
        if point_order_is_given:
            simplices = [frozenset(self.contained_simplex(large=False, point_order=point_order))]
        else:
            simplices = [frozenset(self.contained_simplex(large=True))]
        for s in simplices[0]:
            try:
                point_order.remove(s)
            except ValueError:
                pass
        facets = facets_of_simplex(simplices[0])

        # successively place the remaining points

        # TODO: In concordance with the heuristic to choose a LARGE starting simplex,
        # one could continue to try to pick points that are far away from the previous ones,
        # unless point_order_is_given.
        for point in point_order:
            # identify visible facets
            visible_facets = []
            for facet in facets:
                origin = next(iter(facet))
                normal = facet_normals[facet]
                v = point.reduced_affine_vector() - origin.reduced_affine_vector()
                if v * normal > 0:
                    visible_facets.append(facet)

            # construct simplices over each visible facet
            new_facets = set()
            for facet in visible_facets:
                simplex = frozenset(list(facet) + [point])
                simplices.append(simplex)
                for facet in facets_of_simplex(simplex):
                    if facet in visible_facets:
                        continue
                    if facet in new_facets:
                        new_facets.remove(facet)
                        continue
                    new_facets.add(facet)
            facets.difference_update(visible_facets)
            facets.update(new_facets)

        # construct the triangulation
        triangulation = [[p.index() for p in simplx] for simplx in simplices]
        return self(triangulation)

    pushing_triangulation = placing_triangulation

    @cached_method
    def Gale_transform(self, points=None, homogenize=True):
        r"""
        Return the Gale transform of ``self``.

        INPUT:

        - ``points`` -- tuple of points or point indices or ``None``
          (default). A subset of points for which to compute the Gale
          transform. By default, all points are used.

        - ``homogenize`` -- boolean (default: ``True``); whether to add a row
          of 1's before taking the transform.

        OUTPUT: a matrix over :meth:`base_ring`

        EXAMPLES::

            sage: pc = PointConfiguration([(0,0), (1,0), (2,1), (1,1), (0,1)])
            sage: pc.Gale_transform()
            [ 1 -1  0  1 -1]
            [ 0  0  1 -2  1]

            sage: pc.Gale_transform((0,1,3,4))
            [ 1 -1  1 -1]

            sage: points = (pc.point(0), pc.point(1), pc.point(3), pc.point(4))
            sage: pc.Gale_transform(points)
            [ 1 -1  1 -1]

        It is possible to take the inverse of the Gale transform, by specifying
        whether to homogenize or not::

            sage: pc2 = PointConfiguration([[0,0],[3,0],[0,3],[3,3],[1,1]])
            sage: pc2.Gale_transform(homogenize=False)
            [ 1  0  0  0  0]
            [ 0  1  1  0 -3]
            [ 0  0  0  1 -3]
            sage: pc2.Gale_transform(homogenize=True)
            [ 1  1  1  0 -3]
            [ 0  2  2 -1 -3]

        It might not affect the result (when acyclic)::

            sage: PC = PointConfiguration([[4,0,0],[0,4,0],[0,0,4],[2,1,1],[1,2,1],[1,1,2]])
            sage: GT = PC.Gale_transform(homogenize=False);GT
            [ 1  0  0 -3  1  1]
            [ 0  1  0  1 -3  1]
            [ 0  0  1  1  1 -3]
            sage: GT = PC.Gale_transform(homogenize=True);GT
            [ 1  0  0 -3  1  1]
            [ 0  1  0  1 -3  1]
            [ 0  0  1  1  1 -3]

        The following point configuration is totally cyclic (the cone spanned
        by the vectors is equal to the vector space spanned by the points),
        hence its Gale dual is acyclic (there is a linear functional that is
        positive in all the points of the configuration) when not homogenized::

            sage: pc3 = PointConfiguration([[-1, -1, -1], [-1, 0, 0], [0, -1, 0], [0, 0, -1], [1, 0, 0], [0, 0, 1], [0, 1, 0]])
            sage: g_hom = pc3.Gale_transform(homogenize=True);g_hom
            [ 1  0  0 -2  1 -1  1]
            [ 0  1  0 -1  1 -1  0]
            [ 0  0  1 -1  0 -1  1]
            sage: g_inhom = pc3.Gale_transform(homogenize=False);g_inhom
            [1 0 0 0 1 1 1]
            [0 1 0 0 1 0 0]
            [0 0 1 0 0 0 1]
            [0 0 0 1 0 1 0]
            sage: Polyhedron(rays=g_hom.columns())
            A 3-dimensional polyhedron in ZZ^3 defined as the convex hull of 1 vertex and 3 lines
            sage: Polyhedron(rays=g_inhom.columns())
            A 4-dimensional polyhedron in ZZ^4 defined as the convex hull of 1 vertex and 4 rays
        """
        self._assert_is_affine()
        if points is None:
            points = self.points()
        else:
            try:
                points = [ self.point(ZZ(i)) for i in points ]
            except TypeError:
                pass
        if homogenize:
            m = matrix([(1,) + p.affine() for p in points])
        else:
            m = matrix([p.affine() for p in points])
        return m.left_kernel().matrix()

    def deformation_cone(self, collection):
        r"""
        Return the deformation cone for the ``collection`` of subconfigurations
        of ``self``.

        INPUT:

        - ``collection`` -- a collection of subconfigurations of ``self``.
          Subconfigurations are given as indices

        OUTPUT: a polyhedron. It contains the liftings of the point configuration
        making the collection a regular (or coherent, or projective, or
        polytopal) subdivision.

        EXAMPLES::

            sage: PC = PointConfiguration([(-1, -1), (-1, 0), (0, -1), (1, 0), (0, 1)])
            sage: coll = [(1, 4), (0, 2), (0, 1), (2, 3), (3, 4)]
            sage: dc = PC.deformation_cone(coll);dc
            A 5-dimensional polyhedron in QQ^5 defined as the convex hull of 1 vertex, 3 rays, 2 lines
            sage: dc.rays()
            (A ray in the direction (1, 0, 1, 0, 0),
             A ray in the direction (1, 1, 0, 0, 0),
             A ray in the direction (1, 1, 1, 0, 0))
            sage: dc.lines()
            (A line in the direction (1, 0, 1, 0, -1),
             A line in the direction (1, 1, 0, -1, 0))
            sage: dc.an_element()
            (3, 2, 2, 0, 0)

        We add to the interior element the first line and we verify that the
        given rays are defining rays of the lower hull::

            sage: P = Polyhedron(rays=[(-1, -1, 4), (-1, 0, 3), (0, -1, 2), (1, 0, -1), (0, 1, 0)])
            sage: P.rays()
            (A ray in the direction (-1, -1, 4),
             A ray in the direction (-1, 0, 3),
             A ray in the direction (0, -1, 2),
             A ray in the direction (0, 1, 0),
             A ray in the direction (1, 0, -1))

        Let's verify the mother of all examples explained in Section 7.1.1 of
        [DLRS2010]_::

            sage: def mother(epsilon=0):
            ....:     return PointConfiguration([(4-epsilon,epsilon,0),(0,4-epsilon,epsilon),(epsilon,0,4-epsilon),(2,1,1),(1,2,1),(1,1,2)])

            sage: epsilon = 0
            sage: m = mother(0)
            sage: m.points()
            (P(4, 0, 0), P(0, 4, 0), P(0, 0, 4), P(2, 1, 1), P(1, 2, 1), P(1, 1, 2))
            sage: S1 = [(0,1,4),(0,3,4),(1,2,5),(1,4,5),(0,2,3),(2,3,5)]
            sage: S2 = [(0,1,3),(1,3,4),(1,2,4),(2,4,5),(0,2,5),(0,3,5)]

        Both subdivisions `S1` and `S2` are not regular::

            sage: mother_dc1 = m.deformation_cone(S1)
            sage: mother_dc1
            A 4-dimensional polyhedron in QQ^6 defined as the convex hull of 1 vertex, 1 ray, 3 lines
            sage: mother_dc2 = m.deformation_cone(S2)
            sage: mother_dc2
            A 4-dimensional polyhedron in QQ^6 defined as the convex hull of 1 vertex, 1 ray, 3 lines

        Notice that they have a ray which provides a degenerate lifting which
        only provides a coarsening of the subdivision from the lower hull (it
        has 5 facets, and should have 8)::

            sage: result = Polyhedron([vector(list(m.points()[_])+[mother_dc1.rays()[0][_]]) for _ in range(len(m.points()))])
            sage: result.f_vector()
            (1, 6, 9, 5, 1)

        But if we use epsilon to perturb the configuration, suddenly
        `S1` becomes regular::

            sage: epsilon = 1/2
            sage: mp = mother(epsilon)
            sage: mp.points()
            (P(7/2, 1/2, 0),
             P(0, 7/2, 1/2),
             P(1/2, 0, 7/2),
             P(2, 1, 1),
             P(1, 2, 1),
             P(1, 1, 2))
            sage: mother_dc1 = mp.deformation_cone(S1);mother_dc1
            A 6-dimensional polyhedron in QQ^6 defined as the convex hull of 1 vertex, 3 rays, 3 lines
            sage: mother_dc2 = mp.deformation_cone(S2);mother_dc2
            A 3-dimensional polyhedron in QQ^6 defined as the convex hull of 1 vertex and 3 lines

        .. SEEALSO::

            :meth:`~sage.schemes.toric.variety.Kaehler_cone`

        REFERENCES:

            For more information, see Section 5.4 of [DLRS2010]_ and Section
            2.2 of [ACEP2020].
        """
        from sage.geometry.polyhedron.constructor import Polyhedron
        gale = self.Gale_transform(homogenize=False)
        dual_rays = gale.columns()
        n = self.n_points()
        K = None
        for cone_indices in collection:
            dual_cone = Polyhedron(rays=[dual_rays[i] for i in range(n) if i not in cone_indices])
            K = K.intersection(dual_cone) if K is not None else dual_cone
        preimages = [gale.solve_right(r.vector()) for r in K.rays()]
        return Polyhedron(lines=matrix(self.points()).transpose().rows(),rays=preimages)

    def plot(self, **kwds):
        r"""
        Produce a graphical representation of the point configuration.

        EXAMPLES::

            sage: p = PointConfiguration([[0,0], [0,1], [1,0], [1,1], [-1,-1]])
            sage: p.plot(axes=False)                                                    # needs sage.plot
            Graphics object consisting of 5 graphics primitives

        .. PLOT::
            :width: 300 px

            p = PointConfiguration([[0,0], [0,1], [1,0], [1,1], [-1,-1]])
            sphinx_plot(p.plot(axes=False))
        """
        return self.element_class([], parent=self, check=False).plot(**kwds)



# sage_setup: distribution = sagemath-polyhedra
r"""
Double Description for Arbitrary Polyhedra

This module is part of the python backend for polyhedra. It uses the
double description method for cones
:mod:`~sage.geometry.polyhedron.double_description` to find minimal
H/V-representations of polyhedra. The latter works with cones
only. This is sufficient to treat general polyhedra by the following
construction: Any polyhedron can be embedded in one dimension higher
in the hyperplane `(1, *, \dots, *)`. The cone over the embedded
polyhedron will be called the *homogenized cone* in the
following. Conversely, intersecting the homogenized cone with the
hyperplane `x_0=1` gives you back the original polyhedron.

While slower than specialized C/C++ implementations, the
implementation is general and works with any field in Sage that allows
you to define polyhedra.

.. NOTE::

    If you just want polyhedra over arbitrary fields then you should
    just use the
    :func:`~sage.geometry.polyhedron.constructor.Polyhedron`
    constructor.

EXAMPLES::

    sage: from sage.geometry.polyhedron.double_description_inhomogeneous \
    ....:     import Hrep2Vrep, Vrep2Hrep
    sage: Hrep2Vrep(QQ, 2, [(1,2,3), (2,4,3)], [])
    [-1/2|-1/2  1/2|]
    [   0| 2/3 -1/3|]

Note that the columns of the printed matrix are the vertices, rays,
and lines of the minimal V-representation. Dually, the rows of the
following are the inequalities and equations::

    sage: Vrep2Hrep(QQ, 2, [(-1/2,0)], [(-1/2,2/3), (1/2,-1/3)], [])
    [1 2 3]
    [2 4 3]
    [-----]
"""

# ****************************************************************************
#       Copyright (C) 2014 Volker Braun <vbraun.name@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#                  https://www.gnu.org/licenses/
# ****************************************************************************

from sage.structure.sage_object import SageObject
from sage.matrix.constructor import matrix
from sage.modules.free_module_element import vector
from sage.geometry.polyhedron.double_description import StandardAlgorithm as Algorithm

# Compare with PPL if the base ring is QQ. Can be left enabled since
# we don't use the Python fallback for polyhedra over QQ unless you
# construct one by hand.
VERIFY_RESULT = True


class PivotedInequalities(SageObject):

    def __init__(self, base_ring, dim):
        """
        Base class for inequalities that may contain linear subspaces.

        INPUT:

        - ``base_ring`` -- a field

        - ``dim`` -- integer; the ambient space dimension

        EXAMPLES::

            sage: from sage.geometry.polyhedron.double_description_inhomogeneous \
            ....:     import PivotedInequalities
            sage: piv = PivotedInequalities(QQ, 2)
            sage: piv._pivot_inequalities(matrix([(1,1,3), (5,5,7)]))
            [1 3]
            [5 7]
            sage: piv._pivots
            (0, 2)
            sage: piv._linear_subspace
            Free module of degree 3 and rank 1 over Integer Ring
            Echelon basis matrix:
            [ 1 -1  0]
        """
        self.base_ring = base_ring
        self.dim = dim

    def _pivot_inequalities(self, A):
        """
        Pick pivots for inequalities.

        INPUT:

        - ``A`` -- matrix; the inequalities

        OUTPUT: the matrix of pivot columns

        EXAMPLES::

            sage: from sage.geometry.polyhedron.double_description_inhomogeneous \
            ....:     import PivotedInequalities
            sage: piv = PivotedInequalities(QQ, 2)
            sage: piv._pivot_inequalities(matrix([(1,1,3), (5,5,7)]))
            [1 3]
            [5 7]
        """
        self._linear_subspace = A.right_kernel()
        self._pivots = A.pivots()
        self._nonpivots = A.nonpivots()
        return A.matrix_from_columns(self._pivots)

    def _unpivot_ray(self, ray):
        """
        Undo the pivoting to go back to the original inequalities
        containing a linear subspace.

        INPUT:

        - ``ray`` -- ray in the pivoted coordinates

        OUTPUT: ray in the original coordinates

        EXAMPLES::

            sage: from sage.geometry.polyhedron.double_description_inhomogeneous \
            ....:     import PivotedInequalities
            sage: piv = PivotedInequalities(QQ, 2)
            sage: piv._pivot_inequalities(matrix([(1,1,3), (5,5,7)]))
            [1 3]
            [5 7]
            sage: piv._unpivot_ray([1, 3])
            (1, 0, 3)
        """
        result = [self.base_ring.zero()] * (self.dim + 1)
        for r, i in zip(ray, self._pivots):
            result[i] = r
        return vector(self.base_ring, result)


class Hrep2Vrep(PivotedInequalities):

    def __init__(self, base_ring, dim, inequalities, equations):
        """
        Convert H-representation to a minimal V-representation.

        INPUT:

        - ``base_ring`` -- a field

        - ``dim`` -- integer; the ambient space dimension

        - ``inequalities`` -- list of inequalities; each inequality
          is given as constant term, ``dim`` coefficients

        - ``equations`` -- list of equations; same notation as for
          inequalities

        EXAMPLES::

            sage: from sage.geometry.polyhedron.double_description_inhomogeneous import Hrep2Vrep
            sage: Hrep2Vrep(QQ, 2, [(1,2,3), (2,4,3)], [])
            [-1/2|-1/2  1/2|]
            [   0| 2/3 -1/3|]
            sage: Hrep2Vrep(QQ, 2, [(1,2,3), (2,-2,-3)], [])
            [   1 -1/2||   1]
            [   0    0||-2/3]
            sage: Hrep2Vrep(QQ, 2, [(1,2,3), (2,2,3)], [])
            [-1/2| 1/2|   1]
            [   0|   0|-2/3]
            sage: Hrep2Vrep(QQ, 2, [(8,7,-2), (1,-4,3), (4,-3,-1)], [])
            [ 1  0 -2||]
            [ 1  4 -3||]
            sage: Hrep2Vrep(QQ, 2, [(1,2,3), (2,4,3), (5,-1,-2)], [])
            [-19/5  -1/2| 2/33  1/11|]
            [ 22/5     0|-1/33 -2/33|]
            sage: Hrep2Vrep(QQ, 2, [(0,2,3), (0,4,3), (0,-1,-2)], [])
            [   0| 1/2  1/3|]
            [   0|-1/3 -1/6|]
            sage: Hrep2Vrep(QQ, 2, [], [(1,2,3), (7,8,9)])
            [-2||]
            [ 1||]
            sage: Hrep2Vrep(QQ, 2, [(1,0,0)], [])    # universe
            [0||1 0]
            [0||0 1]
            sage: Hrep2Vrep(QQ, 2, [(-1,0,0)], [])   # empty
            []
            sage: Hrep2Vrep(QQ, 2, [], [])   # universe
            [0||1 0]
            [0||0 1]
        """
        super().__init__(base_ring, dim)
        inequalities = [list(x) for x in inequalities]
        equations = [list(x) for x in equations]
        if not inequalities and not equations:
            # Adding a trivial inequality, so that the ambient dimension is passed to the algorithm.
            inequalities = [[self.base_ring.one()] + [self.base_ring.zero()] * self.dim]
        A = self._init_Vrep(inequalities, equations)
        DD = Algorithm(A).run()
        self._extract_Vrep(DD)
        if VERIFY_RESULT:
            self.verify(inequalities, equations)

    def _init_Vrep(self, inequalities, equations):
        """
        Split off the linear subspace from the inequalities and select pivots.

        INPUT:

        - ``inequalities``, ``equations`` -- see :class:`Vrep2Hrep`

        OUTPUT: the pivoted inequalities

        TESTS::

            sage: from sage.geometry.polyhedron.double_description_inhomogeneous import Hrep2Vrep
            sage: H2V = Hrep2Vrep(QQ, 2, [], [])
            sage: H2V._init_Vrep([(1,0,3), (3,0,1)], [])
            [1 3]
            [3 1]
        """
        neg_eqns = [[-e for e in eqn] for eqn in equations]
        A = matrix(self.base_ring, equations + neg_eqns + inequalities)
        return self._pivot_inequalities(A)

    def _split_linear_subspace(self):
        r"""
        Split the linear subspace in a generator with `x_0\not=0` and the
        remaining generators with `x_0=0`.

        OUTPUT:

        Pair consisting of a line generator with its first coordinate
        scaled to one (if it exists, otherwise ``None``) and a list of
        remaining line generators whose first coordinate has been
        chosen to be zero.

        EXAMPLES::

            sage: from sage.geometry.polyhedron.double_description_inhomogeneous import Hrep2Vrep
            sage: H = Hrep2Vrep(QQ, 2, [(1,2,3)], [])
            sage: H._split_linear_subspace()
            ((1, 0, -1/3), [(0, 1, -2/3)])
            sage: H = Hrep2Vrep(QQ, 2, [(1,0,0)], [])
            sage: H._split_linear_subspace()
            (None, [(0, 1, 0), (0, 0, 1)])
        """
        lines = self._linear_subspace.basis_matrix().rows()
        L0 = []
        L1 = []
        zero = self.base_ring.zero()
        for l in lines:
            if l[0] == zero:
                L0.append(l)
            else:
                l = l / l[0]
                L1.append(l)
        if len(L1) == 0:
            return None, L0
        else:
            l1 = L1.pop()
            return l1, L0 + [l - l[0] * l1 for l in L1]

    def _extract_Vrep(self, DD):
        r"""
        Extract the V-representation from the extremal rays
        of the homogeneous cone.

        The V-representation is the intersection of the cone generated
        by the rays `R` and ``self._linear_subspace`` with the
        hyperplane `(1, *, *, \ldots, *)`.

        INPUT:

        - ``DD`` -- a
          :class:`~sage.geometry.polyhedron.double_description.DoubleDescriptionPair`

        TESTS::

            sage: from sage.geometry.polyhedron.double_description_inhomogeneous import Hrep2Vrep
            sage: H = Hrep2Vrep(QQ, 1, [(1,2)], [])
            sage: from sage.geometry.polyhedron.double_description import StandardAlgorithm
            sage: DD = StandardAlgorithm(matrix([[1,2], [3,5]])).run()
            sage: H._extract_Vrep(DD)
            sage: H.vertices
            [(-1/2)]
        """
        R = [self._unpivot_ray(_) for _ in DD.R]

        line1, L0 = self._split_linear_subspace()
        if line1:
            # we can shift all rays to have x_0 = 0, stay extremal
            L1 = [line1]
            R1 = []
            R0 = [r - r[0] * line1 for r in R]
        else:
            # have to really intersect with x_0 = 0
            L1 = []
            zero = self.base_ring.zero()
            R1 = [r / r[0] for r in R if r[0] > zero]
            DD0 = DD.first_coordinate_plane()
            R0 = [self._unpivot_ray(_) for _ in DD0.R]

        vertices = []
        one = self.base_ring.one()
        for v in R1 + L1:
            assert v[0] == one
            vertices.append(v[1:])
        self.vertices = vertices
        if len(vertices) > 0:
            self.rays = [r[1:] for r in R0]
            self.lines = [l[1:] for l in L0]
        else:
            # empty polyhedron
            self.rays = self.lines = []

    def _repr_(self):
        """
        Return a string representation.

        EXAMPLES::

            sage: from sage.geometry.polyhedron.double_description_inhomogeneous import Hrep2Vrep
            sage: H = Hrep2Vrep(QQ, 1, [(1,2)], [])
            sage: H._repr_()
            '[-1/2| 1/2|]'
        """
        from sage.matrix.constructor import block_matrix

        def make_matrix(rows):
            return matrix(self.base_ring, len(rows), self.dim, rows).transpose()
        V = make_matrix(self.vertices)
        R = make_matrix(self.rays)
        L = make_matrix(self.lines)
        return str(block_matrix([[V, R, L]]))

    def verify(self, inequalities, equations):
        """
        Compare result to PPL if the base ring is QQ.

        This method is for debugging purposes and compares the
        computation with another backend if available.

        INPUT:

        - ``inequalities``, ``equations`` -- see :class:`Hrep2Vrep`

        EXAMPLES::

            sage: from sage.geometry.polyhedron.double_description_inhomogeneous import Hrep2Vrep
            sage: H = Hrep2Vrep(QQ, 1, [(1,2)], [])
            sage: H.verify([(1,2)], [])
        """
        from sage.rings.rational_field import QQ
        from sage.geometry.polyhedron.constructor import Polyhedron
        if self.base_ring is not QQ:
            return
        P = Polyhedron(vertices=self.vertices, rays=self.rays, lines=self.lines,
                       base_ring=QQ, ambient_dim=self.dim, backend='ppl')
        Q = Polyhedron(ieqs=inequalities, eqns=equations,
                       base_ring=QQ, ambient_dim=self.dim, backend='ppl')
        if (P != Q) or \
           (len(self.vertices) != P.n_vertices()) or \
           (len(self.rays) != P.n_rays()) or \
           (len(self.lines) != P.n_lines()):
            print('incorrect!', end="")
            print(Q.Vrepresentation())
            print(P.Hrepresentation())


class Vrep2Hrep(PivotedInequalities):

    def __init__(self, base_ring, dim, vertices, rays, lines):
        """
        Convert V-representation to a minimal H-representation.

        INPUT:

        - ``base_ring`` -- a field

        - ``dim`` -- integer; the ambient space dimension

        - ``vertices`` -- list of vertices; each vertex is given as
          list of ``dim`` coordinates

        - ``rays`` -- list of rays; each ray is given as
          list of ``dim`` coordinates, not all zero

        - ``lines`` -- list of line generators; each line is given as
          list of ``dim`` coordinates, not all zero

        EXAMPLES::

            sage: from sage.geometry.polyhedron.double_description_inhomogeneous import Vrep2Hrep
            sage: Vrep2Hrep(QQ, 2, [(-1/2,0)], [(-1/2,2/3), (1/2,-1/3)], [])
            [1 2 3]
            [2 4 3]
            [-----]

            sage: Vrep2Hrep(QQ, 2, [(1,0), (-1/2,0)], [], [(1,-2/3)])
            [ 1/3  2/3    1]
            [ 2/3 -2/3   -1]
            [--------------]

            sage: Vrep2Hrep(QQ, 2, [(-1/2,0)], [(1/2,0)], [(1,-2/3)])
            [1 2 3]
            [-----]

            sage: Vrep2Hrep(QQ, 2, [(1,1), (0,4), (-2,-3)], [], [])
            [ 8/13  7/13 -2/13]
            [ 1/13 -4/13  3/13]
            [ 4/13 -3/13 -1/13]
            [-----------------]

            sage: Vrep2Hrep(QQ, 2, [(-19/5,22/5), (-1/2,0)], [(2/33,-1/33), (1/11,-2/33)], [])
            [10/11 -2/11 -4/11]
            [ 66/5 132/5  99/5]
            [ 2/11  4/11  6/11]
            [-----------------]

            sage: Vrep2Hrep(QQ, 2, [(0,0)], [(1/2,-1/3), (1/3,-1/6)], [])
            [  0  -6 -12]
            [  0  12  18]
            [-----------]

            sage: Vrep2Hrep(QQ, 2, [(-1/2,0)], [], [(1,-2/3)])
            [-----]
            [1 2 3]

            sage: Vrep2Hrep(QQ, 2, [(-1/2,0)], [], [(1,-2/3), (1,0)])
            []
        """
        super().__init__(base_ring, dim)
        if rays or lines:
            assert len(vertices) > 0
        if not vertices and not rays and not lines:
            # The algorithm does not work, as the ambient dimension cannot be passed.
            # Manually setting a single equality in this case.
            one = self.base_ring.one()
            zero = self.base_ring.zero()
            self.equations = [[one] + [zero]*self.dim]
            self.inequalities = []
        else:
            A = self._init_Vrep(vertices, rays, lines)
            DD = Algorithm(A).run()
            self._extract_Hrep(DD)
        if VERIFY_RESULT:
            self.verify(vertices, rays, lines)

    def _init_Vrep(self, vertices, rays, lines):
        """
        Split off the linear subspace from the inequalities and select pivots.

        INPUT:

        - ``vertices``, ``rays``, ``lines`` -- see :class:`Vrep2Hrep`

        OUTPUT: matrix of pivoted inequalities for the dual homogenized cone

        TESTS::

            sage: from sage.geometry.polyhedron.double_description_inhomogeneous import Vrep2Hrep
            sage: V2H = Vrep2Hrep(QQ, 2, [(-1/2,0)], [(-1/2,2/3), (1/2,-1/3)], [])
            sage: V2H._init_Vrep([(-1/2,0)], [(-1/2,2/3), (1/2,-1/3)], [])
            [   1 -1/2    0]
            [   0 -1/2  2/3]
            [   0  1/2 -1/3]
        """
        one = self.base_ring.one()
        zero = self.base_ring.zero()
        homogeneous = \
            [[one] + list(v) for v in vertices] + \
            [[zero] + list(r) for r in rays] + \
            [[zero] + list(l) for l in lines] + \
            [[zero] + [-x for x in l] for l in lines]
        A = matrix(self.base_ring, homogeneous)
        return self._pivot_inequalities(A)

    def _extract_Hrep(self, DD):
        """
        Extract generators from the dual description of the homogenized cone.

        INPUT:

        - ``DD`` -- a
          :class:`~sage.geometry.polyhedron.double_description.DoubleDescriptionPair`

        EXAMPLES::

            sage: from sage.geometry.polyhedron.double_description_inhomogeneous import Vrep2Hrep
            sage: V2H = Vrep2Hrep(QQ, 1, [(-1/2,), (2/3)], [], [])
            sage: from sage.geometry.polyhedron.double_description import StandardAlgorithm
            sage: DD = StandardAlgorithm(matrix([[1,2], [3,5]])).run()
            sage: V2H._extract_Hrep(DD)
        """
        zero = self.base_ring.zero()

        def is_trivial(ray):
            # trivial Hrep output 1 >= 0
            return ray[0] > zero and all(r == zero for r in ray[1:])
        ieqs = (self._unpivot_ray(ra) for ra in DD.R)
        self.inequalities = [r for r in ieqs if not is_trivial(r)]
        self.equations = self._linear_subspace.matrix().rows()

    def _repr_(self):
        r"""
        Return a string representation.

        OUTPUT: string

        EXAMPLES::

            sage: from sage.geometry.polyhedron.double_description_inhomogeneous import Vrep2Hrep
            sage: V2H = Vrep2Hrep(QQ, 2, [(-1/2,0)], [(-1/2,2/3), (1/2,-1/3)], [])
            sage: V2H._repr_()
            '[1 2 3]\n[2 4 3]\n[-----]'
        """
        from sage.matrix.constructor import block_matrix

        def make_matrix(cols):
            return matrix(self.base_ring, len(cols), self.dim + 1, cols)
        I = make_matrix(self.inequalities)
        E = make_matrix(self.equations)
        return str(block_matrix([[I], [E]]))

    def verify(self, vertices, rays, lines):
        """
        Compare result to PPL if the base ring is QQ.

        This method is for debugging purposes and compares the
        computation with another backend if available.

        INPUT:

        - ``vertices``, ``rays``, ``lines`` -- see :class:`Vrep2Hrep`

        EXAMPLES::

            sage: from sage.geometry.polyhedron.double_description_inhomogeneous import Vrep2Hrep
            sage: vertices = [(-1/2,0)]
            sage: rays = [(-1/2,2/3), (1/2,-1/3)]
            sage: lines = []
            sage: V2H = Vrep2Hrep(QQ, 2, vertices, rays, lines)
            sage: V2H.verify(vertices, rays, lines)
        """
        from sage.rings.rational_field import QQ
        from sage.geometry.polyhedron.constructor import Polyhedron
        if self.base_ring is not QQ:
            return
        P = Polyhedron(vertices=vertices, rays=rays, lines=lines,
                       base_ring=QQ, ambient_dim=self.dim)
        Q = Polyhedron(ieqs=self.inequalities, eqns=self.equations,
                       base_ring=QQ, ambient_dim=self.dim)
        if not P == Q:
            print('incorrect!', P, Q)
            print(Q.Vrepresentation())
            print(P.Hrepresentation())



# sage_setup: distribution = sagemath-polyhedra
r"""
Point collections

This module was designed as a part of framework for toric varieties
(:mod:`~sage.schemes.toric.variety`,
:mod:`~sage.schemes.toric.fano_variety`).

AUTHORS:

- Andrey Novoseltsev (2011-04-25): initial version, based on cone module.

- Andrey Novoseltsev (2012-03-06): additions and doctest changes while
  switching cones to use point collections.

EXAMPLES:

The idea behind :class:`point collections <PointCollection>` is to have a
container for points of the same space that

* behaves like a tuple *without significant performance penalty*::

    sage: c = Cone([(0,0,1), (1,0,1), (0,1,1), (1,1,1)]).rays()
    sage: c[1]
    N(1, 0, 1)
    sage: for point in c: point
    N(0, 0, 1)
    N(1, 0, 1)
    N(0, 1, 1)
    N(1, 1, 1)

* prints in a convenient way and with clear indication of the ambient space::

    sage: c
    N(0, 0, 1),
    N(1, 0, 1),
    N(0, 1, 1),
    N(1, 1, 1)
    in 3-d lattice N

* allows (cached) access to alternative representations::

    sage: c.set()
    frozenset({N(0, 0, 1), N(0, 1, 1), N(1, 0, 1), N(1, 1, 1)})

* allows introduction of additional methods::

    sage: c.basis()
    N(0, 0, 1),
    N(1, 0, 1),
    N(0, 1, 1)
    in 3-d lattice N

Examples of natural point collections include ray and line generators of cones,
vertices and points of polytopes, normals to facets, their subcollections, etc.

Using this class for all of the above cases allows for unified interface *and*
cache sharing. Suppose that `\Delta` is a reflexive polytope. Then the same
point collection can be linked as

#. vertices of `\Delta`;
#. facet normals of its polar `\Delta^\circ`;
#. ray generators of the face fan of `\Delta`;
#. ray generators of the normal fan of `\Delta`.

If all these objects are in use and, say, a matrix representation was computed
for one of them, it becomes available to all others as well, eliminating the
need to spend time and memory four times.
"""

# ***************************************************************************
#       Copyright (C) 2012 Andrey Novoseltsev <novoselt@gmail.com>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#                  https://www.gnu.org/licenses/
# ***************************************************************************

from sage.structure.sage_object cimport SageObject
from sage.structure.richcmp cimport richcmp_not_equal, richcmp

from sage.geometry.toric_lattice import ToricLattice
from sage.matrix.constructor import matrix
from sage.misc.latex import latex


def is_PointCollection(x):
    r"""
    Check if ``x`` is a :class:`point collection <PointCollection>`.

    INPUT:

    - ``x`` -- anything

    OUTPUT: ``True`` if ``x`` is a point collection and ``False`` otherwise

    EXAMPLES::

        sage: from sage.geometry.point_collection import PointCollection
        sage: isinstance(1, PointCollection)
        False
        sage: c = Cone([(0,0,1), (1,0,1), (0,1,1), (1,1,1)])
        sage: isinstance(c.rays(), PointCollection)
        True
    """
    from sage.misc.superseded import deprecation_cython
    deprecation_cython(38126,
                       "The function is_PointCollection is deprecated; "
                       "use 'isinstance(..., PointCollection)' instead.")
    return isinstance(x, PointCollection)


_output_format = "default"


cdef class PointCollection(SageObject):
    r"""
    Create a point collection.

    .. WARNING::

        No correctness check or normalization is performed on the input data.
        This class is designed for internal operations and you probably should
        not use it directly.

    Point collections are immutable, but cache most of the returned values.

    INPUT:

    - ``points`` -- an iterable structure of immutable elements of ``module``,
      if ``points`` are already accessible to you as a :class:`tuple`, it is
      preferable to use it for speed and memory consumption reasons;

    - ``module`` -- an ambient module for ``points``. If ``None`` (the default),
      it will be determined as :func:`parent` of the first point. Of course, this
      cannot be done if there are no points, so in this case you must give an
      appropriate ``module`` directly.

    OUTPUT:

    - a point collection.
    """

    cdef:
        tuple _points
        object _module
        # cache attributes
        PointCollection _basis
        object _matrix
        frozenset _set

    def __init__(self, points, module=None):
        r"""
        See :class:`PointCollection` for documentation.

        TESTS::

            sage: from sage.geometry.point_collection import PointCollection
            sage: v = vector([1,0])
            sage: v.set_immutable()
            sage: c = PointCollection([v], ZZ^2)
            sage: c.module()
            Ambient free module of rank 2
            over the principal ideal domain Integer Ring
            sage: c = PointCollection([v], None)
            sage: c.module()  # Determined automatically
            Ambient free module of rank 2
            over the principal ideal domain Integer Ring
            sage: TestSuite(c).run()
        """
        super().__init__()
        self._points = tuple(points)
        self._module = self._points[0].parent() if module is None else module

    def _sage_input_(self, sib, coerced):
        r"""
        Return Sage command to reconstruct ``self``.

        See :mod:`sage.misc.sage_input` for details.

        EXAMPLES::

            sage: c = Cone([(0,0,1), (1,0,1), (0,1,1), (1,1,1)]).rays()
            sage: sage_input(c, verify=True)
            # Verified
            sage.geometry.point_collection.PointCollection((vector(ZZ, [0, 0, 1]), vector(ZZ, [1, 0, 1]), vector(ZZ, [0, 1, 1]), vector(ZZ, [1, 1, 1])))

            sage: c = sage.geometry.point_collection.PointCollection([], ToricLattice(2, 'U'))
            sage: sage_input(c, verify=True)
            # Verified
            sage.geometry.point_collection.PointCollection((), ToricLattice(2, 'U', 'U*', 'U', 'U^*'))
        """
        args = [sib(self._points)]
        if not self._points or self._module is not self._points[0].parent():
            args.append(sib(self._module))
        return sib.name('sage.geometry.point_collection.PointCollection')(*args)

    def __add__(left, right):
        r"""
        Return the joint point collection.

        INPUT:

        - ``left`` -- a :class:`PointCollection`;

        - ``right`` -- a :class:`PointCollection`

        OUTPUT: a :class:`PointCollection`

        TESTS::

            sage: c = Cone([(0,0,1), (1,0,1), (0,1,1), (1,1,1)]).rays()
            sage: c + c
            N(0, 0, 1),
            N(1, 0, 1),
            N(0, 1, 1),
            N(1, 1, 1),
            N(0, 0, 1),
            N(1, 0, 1),
            N(0, 1, 1),
            N(1, 1, 1)
            in 3-d lattice N
        """
        if not (isinstance(left, PointCollection) and
                isinstance(right, PointCollection)):
            raise NotImplementedError
        cdef PointCollection left_pc = left
        cdef PointCollection right_pc = right
        if left_pc._module is not right_pc._module:
            raise NotImplementedError
        return PointCollection(left_pc._points + right_pc._points,
                               left_pc._module)

    def __call__(self, *args):
        r"""
        Return a subcollection of ``self``.

        INPUT:

        - a list of integers (as a single or many arguments).

        OUTPUT: a :class:`point collection <PointCollection>`

        TESTS::

            sage: c = Cone([(0,0,1), (1,0,1), (0,1,1), (1,1,1)]).rays()
            sage: c()
            Empty collection
            in 3-d lattice N
            sage: c(2,1)
            N(0, 1, 1),
            N(1, 0, 1)
            in 3-d lattice N
            sage: c(range(4)) == c
            True
        """
        if len(args) == 1:
            try:
                args = tuple(args[0])
            except TypeError:
                pass
        # Avoid creating a copy of self
        if len(args) == len(self) and args == tuple(range(len(self))):
            return self
        else:
            return PointCollection([self[i] for i in args], self._module)

    def __richcmp__(self, right, op):
        r"""
        Compare ``self`` and ``right`` according to the operator ``op``.

        INPUT:

        - ``right`` -- another PointCollection

        OUTPUT: boolean

        First compare according to the underlying :meth:`module`
        and then according to the list of points.

        TESTS::

            sage: c = Cone([(0,0,1), (1,0,1), (0,1,1), (1,1,1)]).rays()
            sage: d = Cone([(0,1,2), (1,0,1), (0,1,1), (1,1,1)]).rays()
            sage: c == c
            True
            sage: c == d
            False
        """
        cdef PointCollection left_pc, right_pc
        try:
            left_pc = <PointCollection?>self
            right_pc = <PointCollection?>right
        except TypeError:
            return NotImplemented

        left_m = left_pc._module
        right_m = right_pc._module
        if left_m != right_m:
            return richcmp_not_equal(left_m, right_m, op)
        return richcmp(left_pc._points, right_pc._points, op)

    def __getitem__(self, n):
        r"""
        Return the ``n``-th point of ``self``.

        INPUT:

        - ``n`` -- integer

        OUTPUT: a point, an element of the ambient :meth:`module` of ``self``

        EXAMPLES::

            sage: c = Cone([(0,0,1), (1,0,1), (0,1,1), (1,1,1)]).rays()
            sage: c[0]
            N(0, 0, 1)
        """
        return self._points[n]

    def __hash__(self):
        r"""
        Return the hash of ``self``.

        OUTPUT: integer

        TESTS::

            sage: c = Cone([(0,0,1), (1,0,1), (0,1,1), (1,1,1)]).rays()
            sage: hash(c) == hash(c)
            True
        """
        return hash(self._points)

    def __iter__(self):
        r"""
        Return an iterator over points of ``self``.

        OUTPUT: an iterator

        TESTS::

            sage: c = Cone([(0,0,1), (1,0,1), (0,1,1), (1,1,1)]).rays()
            sage: for point in c: print(point)
            N(0, 0, 1)
            N(1, 0, 1)
            N(0, 1, 1)
            N(1, 1, 1)
        """
        return iter(self._points)

    def __len__(self):
        r"""
        Return the number of points in ``self``.

        OUTPUT: integer

        EXAMPLES::

            sage: c = Cone([(0,0,1), (1,0,1), (0,1,1), (1,1,1)]).rays()
            sage: len(c)
            4
        """
        return len(self._points)

    def __list__(self):
        r"""
        Return a list of points of ``self``.

        OUTPUT: list

        TESTS::

            sage: c = Cone([(0,0,1), (1,0,1), (0,1,1), (1,1,1)]).rays()
            sage: list(c)
            [N(0, 0, 1), N(1, 0, 1), N(0, 1, 1), N(1, 1, 1)]
        """
        return list(self._points)

    def __mul__(left, right):
        r"""
        Return the product ``left * right``.

        INPUT:

        - a :class:`point collection <PointCollection>` and something that can
          act both on ``self.module().zero()`` and either ``self.matrix()`` from
          the right or ``self.column_matrix()`` from the left.

        OUTPUT:

        - the result of ``self.matrix() * right``, provided that
          ``self.module().zero() * right`` can be computed.

        The idea of this method is to provide a shortcut for matrix
        multiplication with appropriate type checks, in particular, it is not
        possible to multiply by a point of the same toric lattice as elements of
        ``self``.

        TESTS::

            sage: c = Cone([(0,0,1), (1,0,1), (0,1,1), (1,1,1)]).rays()
            sage: c.matrix()
            [0 0 1]
            [1 0 1]
            [0 1 1]
            [1 1 1]
            sage: c * c[0]
            Traceback (most recent call last):
            ...
            TypeError: elements of the same toric lattice cannot be multiplied!

        If you really need such a product, state it explicitly::

            sage: c.matrix() * c[0]
            (1, 1, 1, 1)

        Multiplication by matrices works as well::

            sage: c * c.column_matrix()
            [1 1 1 1]
            [1 2 1 2]
            [1 1 2 2]
            [1 2 2 3]
        """
        cdef PointCollection pc
        if isinstance(left, PointCollection):
            pc = left
            # Check that it is possible to act on points.
            pc._module.zero() * right
            return pc.matrix() * right
        if isinstance(right, PointCollection):
            pc = right
            # Check that it is possible to act on points.
            left * pc._module.zero()
            return left * pc.column_matrix()
        raise NotImplementedError

    def __reduce__(self):
        r"""
        Prepare ``self`` for pickling.

        OUTPUT:

        - a tuple, currently the class name and a tuple consisting of points
          and the ambient module.

        TESTS::

            sage: c = Cone([(0,0,1), (1,0,1), (0,1,1), (1,1,1)]).rays()
            sage: loads(dumps(c))
            N(0, 0, 1),
            N(1, 0, 1),
            N(0, 1, 1),
            N(1, 1, 1)
            in 3-d lattice N
            sage: loads(dumps(c)) == c
            True
        """
        return (PointCollection, (self._points, self._module))

    def __tuple__(self):
        r"""
        Return the tuple of points of ``self``.

        OUTPUT: a tuple

        TESTS::

            sage: c = Cone([(0,0,1), (1,0,1), (0,1,1), (1,1,1)]).rays()
            sage: tuple(c)
            (N(0, 0, 1), N(1, 0, 1), N(0, 1, 1), N(1, 1, 1))
        """
        return self._points

    def _latex_(self):
        r"""
        Return a LaTeX representation of ``self``.

        OUTPUT: string

        TESTS::

            sage: c = Cone([(0,0,1), (1,0,1), (0,1,1), (1,1,1)]).rays()
            sage: print(c._latex_())
            \left(\left(0,\,0,\,1\right)_{N}, \left(1,\,0,\,1\right)_{N},
            \left(0,\,1,\,1\right)_{N}, \left(1,\,1,\,1\right)_{N}\right)_{N}
        """
        global _output_format
        if _output_format in ["default", "tuple"]:
            r = latex(tuple(self))
        elif _output_format == "matrix":
            r = latex(self.matrix())
        elif _output_format == "column matrix":
            r = latex(self.column_matrix())
        elif _output_format == "separated column matrix":
            r = latex(self.column_matrix())
            r = r.replace("r" * len(self), "|".join("r" * len(self)))
        return r"%s_{%s}" % (r, latex(self.module()))

    def _matrix_(self, ring=None):
        r"""
        Return a matrix whose rows are points of ``self``.

        INPUT:

        - ``ring`` -- a base ring for the returned matrix (default: base ring of
          :meth:`module` of ``self``)

        OUTPUT: a :class:`matrix <Matrix>`

        EXAMPLES::

            sage: c = Cone([(0,0,1), (1,0,1), (0,1,1), (1,1,1)]).rays()
            sage: matrix(c) # indirect doctest
            [0 0 1]
            [1 0 1]
            [0 1 1]
            [1 1 1]
        """
        if ring is None:
            return self.matrix()
        else:
            return self.matrix().change_ring(ring)

    def _repr_(self):
        r"""
        Return a string representation of ``self``.

        OUTPUT: string

        TESTS::

            sage: c = Cone([(0,0,1), (1,0,1), (0,1,1), (1,1,1)]).rays()
            sage: print(c._repr_())
            N(0, 0, 1),
            N(1, 0, 1),
            N(0, 1, 1),
            N(1, 1, 1)
            in 3-d lattice N
        """
        global _output_format
        if _output_format == "default":
            r = map(repr, self)
            r = [point.split(",") for point in r]
            if not r:
                r = "Empty collection"
            else:
                if "(" in r[0][0]:
                    delimiter = "("
                elif "[" in r[0][0]:
                    delimiter = "["
                else:
                    raise ValueError("cannot parse point representation!")
                heads = []
                for point in r:
                    head, point[0] = point[0].rsplit(delimiter, 1)
                    heads.append(head + delimiter)
                format = "{{:<{}}}".format(max(map(len, heads)))
                widths = [0] * len(r[0])
                for point in r:
                    for i, coordinate in enumerate(point):
                        widths[i] = max(widths[i], len(coordinate))
                format += ",".join("{{:>{}}}".format(width) for width in widths)
                r = ",\n".join(format.format(head, *point)
                               for head, point in zip(heads, r))
        elif _output_format == "tuple":
            r = tuple(self)
        elif _output_format == "matrix":
            r = self.matrix()
        else:
            r = self.column_matrix()
        return "{}\nin {}".format(r, self.module())

    def basis(self):
        r"""
        Return a linearly independent subset of points of ``self``.

        OUTPUT:

        - a :class:`point collection <PointCollection>` giving a random (but
          fixed) choice of an `\RR`-basis for the vector space spanned by the
          points of ``self``.

        EXAMPLES::

            sage: c = Cone([(0,0,1), (1,0,1), (0,1,1), (1,1,1)]).rays()
            sage: c.basis()
            N(0, 0, 1),
            N(1, 0, 1),
            N(0, 1, 1)
            in 3-d lattice N

        Calling this method twice will always return *exactly the same* point
        collection::

            sage: c.basis().basis() is c.basis()
            True
        """
        if self._basis is None:
            self._basis = self(self.matrix().pivot_rows())
        return self._basis

    def cardinality(self):
        r"""
        Return the number of points in ``self``.

        OUTPUT: integer

        EXAMPLES::

            sage: c = Cone([(0,0,1), (1,0,1), (0,1,1), (1,1,1)]).rays()
            sage: c.cardinality()
            4
        """
        return len(self._points)

    def cartesian_product(self, other, module=None):
        r"""
        Return the Cartesian product of ``self`` with ``other``.

        INPUT:

        - ``other`` -- a :class:`point collection <PointCollection>`;

        - ``module`` -- (optional) the ambient module for the result. By
          default, the direct sum of the ambient modules of ``self`` and
          ``other`` is constructed.

        OUTPUT: a :class:`point collection <PointCollection>`

        EXAMPLES::

            sage: c = Cone([(0,0,1), (1,1,1)]).rays()
            sage: c.cartesian_product(c)
            N+N(0, 0, 1, 0, 0, 1),
            N+N(1, 1, 1, 0, 0, 1),
            N+N(0, 0, 1, 1, 1, 1),
            N+N(1, 1, 1, 1, 1, 1)
            in 6-d lattice N+N
        """
        assert isinstance(other, PointCollection)
        if module is None:
            module = self._module.direct_sum(other.module())
        P = [list(p) for p in self]
        Q = [list(q) for q in other]
        PQ = [module(p + q) for q in Q for p in P]
        for pq in PQ:
            pq.set_immutable()
        return PointCollection(PQ, module)

    def column_matrix(self):
        r"""
        Return a matrix whose columns are points of ``self``.

        OUTPUT: a :class:`matrix <Matrix>`

        EXAMPLES::

            sage: c = Cone([(0,0,1), (1,0,1), (0,1,1), (1,1,1)]).rays()
            sage: c.column_matrix()
            [0 1 0 1]
            [0 0 1 1]
            [1 1 1 1]
        """
        return self.matrix().transpose()

    def dimension(self):
        r"""
        Return the dimension of the space spanned by points of ``self``.

        .. NOTE:: You can use either :meth:`dim` or :meth:`dimension`.

        OUTPUT: integer

        EXAMPLES::

            sage: c = Cone([(0,0,1), (1,1,1)]).rays()
            sage: c.dimension()
            2
            sage: c.dim()
            2
        """
        return self.matrix().rank()

    dim = dimension

    def dual_module(self):
        r"""
        Return the dual of the ambient module of ``self``.

        OUTPUT:

        - a :class:`module <FreeModule_generic>`. If possible (that is, if the
          ambient :meth:`module` `M` of ``self`` has a ``dual()`` method), the
          dual module is returned. Otherwise, `R^n` is returned, where `n` is
          the dimension of `M` and `R` is its base ring.

        EXAMPLES::

            sage: c = Cone([(0,0,1), (1,0,1), (0,1,1), (1,1,1)]).rays()
            sage: c.dual_module()
            3-d lattice M
        """
        M = self._module
        try:
            return M.dual()
        except AttributeError:
            # TODO: add support for torsion modules as well?
            return M.base_ring() ** M.dimension()

    def index(self, *args):
        r"""
        Return the index of the first occurrence of ``point`` in ``self``.

        INPUT:

        - ``point`` -- a point of ``self``

        - ``start`` -- (optional) an integer, if given, the search will start
          at this position

        - ``stop`` -- (optional) an integer, if given, the search will stop
          at this position

        OUTPUT: an integer if ``point`` is in ``self[start:stop]``, otherwise a
        :exc:`ValueError` exception is raised

        EXAMPLES::

            sage: c = Cone([(0,0,1), (1,0,1), (0,1,1), (1,1,1)]).rays()
            sage: c.index((0,1,1))
            Traceback (most recent call last):
            ...
            ValueError: tuple.index(x): x not in tuple

        Note that this was not a mistake: the *tuple* ``(0,1,1)`` is *not* a
        point of ``c``! We need to pass actual element of the ambient module of
        ``c`` to get their indices::

            sage: N = c.module()
            sage: c.index(N(0,1,1))
            2
            sage: c[2]
            N(0, 1, 1)
        """
        return self._points.index(*args)

    def matrix(self):
        r"""
        Return a matrix whose rows are points of ``self``.

        OUTPUT: a :class:`matrix <Matrix>`

        EXAMPLES::

            sage: c = Cone([(0,0,1), (1,0,1), (0,1,1), (1,1,1)]).rays()
            sage: c.matrix()
            [0 0 1]
            [1 0 1]
            [0 1 1]
            [1 1 1]
        """
        if self._matrix is None:
            M = matrix(self._module.base_ring(), len(self._points),
                       self._module.degree(), self._points)
            M.set_immutable()
            self._matrix = M
        return self._matrix

    def module(self):
        r"""
        Return the ambient module of ``self``.

        OUTPUT: a :class:`module <FreeModule_generic>`

        EXAMPLES::

            sage: c = Cone([(0,0,1), (1,0,1), (0,1,1), (1,1,1)]).rays()
            sage: c.module()
            3-d lattice N
        """
        return self._module

    @staticmethod
    def output_format(format=None):
        r"""
        Return or set the output format for **ALL** point collections.

        INPUT:

        - ``format`` -- (optional) if given, must be one of the strings
            * "default" -- output one point per line with vertical alignment of
              coordinates in text mode, same as "tuple" for LaTeX;
            * "tuple" -- output ``tuple(self)`` with lattice information;
            * "matrix" -- output :meth:`matrix` with lattice information;
            * "column matrix" -- output :meth:`column_matrix` with lattice
              information;
            * "separated column matrix" -- same as "column matrix" for text
              mode, for LaTeX separate columns by lines (not shown by jsMath).

        OUTPUT:

        - a string with the current format (only if ``format`` was omitted).

        This function affects both regular and LaTeX output.

        EXAMPLES::

            sage: c = Cone([(0,0,1), (1,0,1), (0,1,1), (1,1,1)]).rays()
            sage: c
            N(0, 0, 1),
            N(1, 0, 1),
            N(0, 1, 1),
            N(1, 1, 1)
            in 3-d lattice N
            sage: c.output_format()
            'default'
            sage: c.output_format("tuple")
            sage: c
            (N(0, 0, 1), N(1, 0, 1), N(0, 1, 1), N(1, 1, 1))
            in 3-d lattice N
            sage: c.output_format("matrix")
            sage: c
            [0 0 1]
            [1 0 1]
            [0 1 1]
            [1 1 1]
            in 3-d lattice N
            sage: c.output_format("column matrix")
            sage: c
            [0 1 0 1]
            [0 0 1 1]
            [1 1 1 1]
            in 3-d lattice N
            sage: c.output_format("separated column matrix")
            sage: c
            [0 1 0 1]
            [0 0 1 1]
            [1 1 1 1]
            in 3-d lattice N

        Note that the last two outputs are identical, separators are only
        inserted in the LaTeX mode::

            sage: latex(c)
            \left(\begin{array}{r|r|r|r}
            0 & 1 & 0 & 1 \\
            0 & 0 & 1 & 1 \\
            1 & 1 & 1 & 1
            \end{array}\right)_{N}

        Since this is a static method, you can call it for the class directly::

            sage: from sage.geometry.point_collection import PointCollection
            sage: PointCollection.output_format("default")
            sage: c
            N(0, 0, 1),
            N(1, 0, 1),
            N(0, 1, 1),
            N(1, 1, 1)
            in 3-d lattice N
        """
        global _output_format
        if format is None:
            return _output_format
        assert format in ["default", "tuple", "matrix", "column matrix",
                          "separated column matrix"]
        _output_format = format

    def set(self):
        r"""
        Return points of ``self`` as a :class:`frozenset`.

        OUTPUT: a :class:`frozenset`

        EXAMPLES::

            sage: c = Cone([(0,0,1), (1,0,1), (0,1,1), (1,1,1)]).rays()
            sage: c.set()
            frozenset({N(0, 0, 1), N(0, 1, 1), N(1, 0, 1), N(1, 1, 1)})
        """
        if self._set is None:
            self._set = frozenset(self._points)
        return self._set

    def write_for_palp(self, f):
        r"""
        Write ``self`` into an open file ``f`` in PALP format.

        INPUT:

        - ``f`` -- a file opened for writing

        EXAMPLES::

            sage: o = lattice_polytope.cross_polytope(3)
            sage: from io import StringIO
            sage: f = StringIO()
            sage: o.vertices().write_for_palp(f)
            sage: print(f.getvalue())
            6 3
            1 0 0
            0 1 0
            0 0 1
            -1 0 0
            0 -1 0
            0 0 -1
        """
        f.write('{} {}\n'.format(len(self), self._module.rank()))
        f.write('\n'.join(' '.join(str(c) for c in p) for p in self))
        f.write('\n')


def read_palp_point_collection(f, lattice=None, permutation=False):
    r"""
    Read and return a point collection from an opened file.

    Data must be in PALP format:

        * the first input line starts with two integers `m` and `n`, the number
          of points and the number of components of each;

        * the rest of the first line may contain a permutation;

        * the next `m` lines contain `n` numbers each.

    .. NOTE::

        If `m` < `n`, it is assumed (for compatibility with PALP) that the
        matrix is transposed, i.e. that each column is a point.

    INPUT:

    - ``f`` -- an opened file with PALP output

    - ``lattice`` -- the lattice for points. If not given, the
      :class:`toric lattice <sage.geometry.toric_lattice.ToricLatticeFactory>`
      `M` of dimension `n` will be used.

    - ``permutation`` -- boolean (default: ``False``); if ``True``, try to retrieve
      the permutation. This parameter makes sense only when PALP computed the
      normal form of a lattice polytope.

    OUTPUT:

    - a :class:`point collection <PointCollection>`, optionally followed by
      a permutation. ``None`` if EOF is reached.

    EXAMPLES::

        sage: data = "3 2 regular\n1 2\n3 4\n5 6\n2 3 transposed\n1 2 3\n4 5 6"
        sage: print(data)
        3 2 regular
        1 2
        3 4
        5 6
        2 3 transposed
        1 2 3
        4 5 6
        sage: from io import StringIO
        sage: f = StringIO(data)
        sage: from sage.geometry.point_collection \
        ....:     import read_palp_point_collection
        sage: read_palp_point_collection(f)
        M(1, 2),
        M(3, 4),
        M(5, 6)
        in 2-d lattice M
        sage: read_palp_point_collection(f)
        M(1, 4),
        M(2, 5),
        M(3, 6)
        in 2-d lattice M
        sage: read_palp_point_collection(f) is None
        True
    """
    cdef int i, j, m, n
    first_line = f.readline()
    if first_line == "":
        return None
    first_line = first_line.split()
    m = int(first_line[0])
    n = int(first_line[1])
    if m >= n:
        # Typical situation: a point on each line
        lattice = lattice or ToricLattice(n).dual()
        points = [lattice.element_class(lattice, f.readline().split())
                  for i in range(m)]
    else:
        # Also may appear as PALP output, e.g. points of 3-d polytopes
        lattice = lattice or ToricLattice(m).dual()
        data = [f.readline().split() for j in range(m)]
        points = [lattice.element_class(lattice, [data[j][i] for j in range(m)])
                  for i in range(n)]
    for p in points:
        p.set_immutable()
    pc = PointCollection(points, lattice)
    if permutation:
        last_piece = first_line[-1].split('=')
        if last_piece[0] != 'perm':
            raise ValueError('permutation was requested but not found')
        from sage.geometry.lattice_polytope import _palp_convert_permutation
        p = _palp_convert_permutation(last_piece[1])
        return (pc, p)
    else:
        return pc
